from typing import Any, Optional

from flet.core.alignment import Axis
from flet.core.control import Control
from flet.core.ref import Ref
from flet.core.types import OptionalControlEventCallable


class Draggable(Control):
    """
    A control that can be dragged from to a `DragTarget`.

    When a draggable control recognizes the start of a drag gesture, it displays a `content_feedback` control that tracks the user's finger across the screen. If the user lifts their finger while on top of a `DragTarget`, that target is given the opportunity to complete drag-and-drop flow.

    Example:
    ```
    import flet
    from flet import (
        Column,
        Container,
        Draggable,
        DragTarget,
        DragTargetAcceptEvent,
        Page,
        Row,
        border,
        colors,
    )


    def main(page: Page):
        page.title = "Drag and Drop example"

        def drag_will_accept(e):
            e.control.content.border = border.all(
                2, colors.BLACK45 if e.data == "true" else colors.RED
            )
            e.control.update()

        def drag_accept(e: DragTargetAcceptEvent):
            src = page.get_control(e.src_id)
            e.control.content.bgcolor = src.content.bgcolor
            e.control.content.border = None
            e.control.update()

        def drag_leave(e):
            e.control.content.border = None
            e.control.update()

        page.add(
            Row(
                [
                    Column(
                        [
                            Draggable(
                                group="color",
                                content=Container(
                                    width=50,
                                    height=50,
                                    bgcolor=colors.CYAN,
                                    border_radius=5,
                                ),
                                content_feedback=Container(
                                    width=20,
                                    height=20,
                                    bgcolor=colors.CYAN,
                                    border_radius=3,
                                ),
                            ),
                            Draggable(
                                group="color",
                                content=Container(
                                    width=50,
                                    height=50,
                                    bgcolor=colors.YELLOW,
                                    border_radius=5,
                                ),
                            ),
                            Draggable(
                                group="color1",
                                content=Container(
                                    width=50,
                                    height=50,
                                    bgcolor=colors.GREEN,
                                    border_radius=5,
                                ),
                            ),
                        ]
                    ),
                    Container(width=100),
                    DragTarget(
                        group="color",
                        content=Container(
                            width=50,
                            height=50,
                            bgcolor=colors.BLUE_GREY_100,
                            border_radius=5,
                        ),
                        on_will_accept=drag_will_accept,
                        on_accept=drag_accept,
                        on_leave=drag_leave,
                    ),
                ]
            )
        )


    flet.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/draggable
    """

    def __init__(
        self,
        content: Control,
        group: Optional[str] = None,
        content_when_dragging: Optional[Control] = None,
        content_feedback: Optional[Control] = None,
        axis: Optional[Axis] = None,
        affinity: Optional[Axis] = None,
        max_simultaneous_drags: Optional[int] = None,
        on_drag_start: OptionalControlEventCallable = None,
        on_drag_complete: OptionalControlEventCallable = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        disabled: Optional[bool] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):

        Control.__init__(
            self,
            ref=ref,
            disabled=disabled,
            visible=visible,
            data=data,
        )

        self.group = group
        self.content = content
        self.content_when_dragging = content_when_dragging
        self.content_feedback = content_feedback
        self.on_drag_start = on_drag_start
        self.on_drag_complete = on_drag_complete
        self.axis = axis
        self.affinity = affinity
        self.max_simultaneous_drags = max_simultaneous_drags

    def _get_control_name(self):
        return "draggable"

    def _get_children(self):
        self.__content._set_attr_internal("n", "content")
        children = [self.__content]
        if self.__content_when_dragging:
            self.__content_when_dragging._set_attr_internal(
                "n", "content_when_dragging"
            )
            children.append(self.__content_when_dragging)
        if self.__content_feedback:
            self.__content_feedback._set_attr_internal("n", "content_feedback")
            children.append(self.__content_feedback)
        return children

    def before_update(self):
        super().before_update()
        assert self.__content.visible, "content must be visible"

    # group
    @property
    def group(self) -> Optional[str]:
        return self._get_attr("group")

    @group.setter
    def group(self, value: Optional[str]):
        self._set_attr("group", value)

    # content
    @property
    def content(self) -> Control:
        return self.__content

    @content.setter
    def content(self, value: Control):
        self.__content = value

    # content_when_dragging
    @property
    def content_when_dragging(self) -> Optional[Control]:
        return self.__content_when_dragging

    @content_when_dragging.setter
    def content_when_dragging(self, value: Optional[Control]):
        self.__content_when_dragging = value

    # content_feedback
    @property
    def content_feedback(self) -> Optional[Control]:
        return self.__content_feedback

    @content_feedback.setter
    def content_feedback(self, value: Optional[Control]):
        self.__content_feedback = value

    # max_simultaneous_drags
    @property
    def max_simultaneous_drags(self) -> Optional[int]:
        return self._get_attr("maxSimultaneousDrags")

    @max_simultaneous_drags.setter
    def max_simultaneous_drags(self, value: Optional[int]):
        self._set_attr("maxSimultaneousDrags", value)

    # axis
    @property
    def axis(self) -> Optional[Axis]:
        return self.__axis

    @axis.setter
    def axis(self, value: Optional[Axis]):
        self.__axis = value
        self._set_enum_attr("axis", value, Axis)

    # affinity
    @property
    def affinity(self) -> Optional[Axis]:
        return self.__affinity

    @affinity.setter
    def affinity(self, value: Optional[Axis]):
        self.__affinity = value
        self._set_enum_attr("affinity", value, Axis)

    # on_drag_start
    @property
    def on_drag_start(self) -> OptionalControlEventCallable:
        return self._get_event_handler("dragStart")

    @on_drag_start.setter
    def on_drag_start(self, handler: OptionalControlEventCallable):
        self._add_event_handler("dragStart", handler)

    # on_drag_complete
    @property
    def on_drag_complete(self) -> OptionalControlEventCallable:
        return self._get_event_handler("dragComplete")

    @on_drag_complete.setter
    def on_drag_complete(self, handler: OptionalControlEventCallable):
        self._add_event_handler("dragComplete", handler)



from typing import Any, Optional, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    OffsetValue,
    OptionalControlEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
)


class CupertinoActivityIndicator(ConstrainedControl):
    """
    An iOS-style activity indicator that spins clockwise.

    -----

    Online docs: https://flet.dev/docs/controls/cupertinoactivityindicator
    """

    def __init__(
        self,
        radius: OptionalNumber = None,
        color: Optional[ColorValue] = None,
        animating: Optional[bool] = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        self.color = color
        self.radius = radius
        self.animating = animating

    def _get_control_name(self):
        return "cupertinoactivityindicator"

    # color
    @property
    def color(self) -> Optional[ColorValue]:
        return self.__color

    @color.setter
    def color(self, value: Optional[ColorValue]):
        self.__color = value
        self._set_enum_attr("color", value, ColorEnums)

    # animating
    @property
    def animating(self) -> bool:
        return self._get_attr("animating", data_type="bool", def_value=True)

    @animating.setter
    def animating(self, value: Optional[bool]):
        self._set_attr("animating", value)

    # radius
    @property
    def radius(self) -> OptionalNumber:
        return self._get_attr("radius", data_type="float")

    @radius.setter
    def radius(self, value: OptionalNumber):
        self._set_attr("radius", value)



from typing import Any, List, Optional, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.box import BoxShadow
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    BlendMode,
    ColorEnums,
    ColorValue,
    IconEnums,
    IconValue,
    OffsetValue,
    OptionalControlEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
)


class Icon(ConstrainedControl):
    """
    Displays a Material icon.

    Icon browser: https://flet-icons-browser.fly.dev/#/

    Example:
    ```
    import flet as ft

    def main(page: ft.Page):
        page.add(
            ft.Row(
                [
                    ft.Icon(name=ft.icons.FAVORITE, color=ft.colors.PINK),
                    ft.Icon(name=ft.icons.AUDIOTRACK, color=ft.colors.GREEN_400, size=30),
                    ft.Icon(name=ft.icons.BEACH_ACCESS, color=ft.colors.BLUE, size=50),
                    ft.Icon(name="settings", color="#c1c1c1"),
                ]
            )
        )

    ft.app(target=main)

    ```

    -----

    Online docs: https://flet.dev/docs/controls/icon
    """

    def __init__(
        self,
        name: Optional[IconValue] = None,
        color: Optional[ColorValue] = None,
        size: OptionalNumber = None,
        semantics_label: Optional[str] = None,
        shadows: Union[BoxShadow, List[BoxShadow], None] = None,
        fill: OptionalNumber = None,
        apply_text_scaling: Optional[bool] = None,
        grade: OptionalNumber = None,
        weight: OptionalNumber = None,
        optical_size: OptionalNumber = None,
        blend_mode: Optional[BlendMode] = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        self.name = name
        self.color = color
        self.size = size
        self.semantics_label = semantics_label
        self.shadows = shadows
        self.fill = fill
        self.apply_text_scaling = apply_text_scaling
        self.grade = grade
        self.weight = weight
        self.optical_size = optical_size
        self.blend_mode = blend_mode

    def _get_control_name(self):
        return "icon"

    def before_update(self):
        super().before_update()
        self._set_attr_json("shadows", self.__shadows)

    # name
    @property
    def name(self) -> Optional[IconValue]:
        return self.__name

    @name.setter
    def name(self, value: Optional[IconValue]):
        self.__name = value
        self._set_enum_attr("name", value, IconEnums)

    # blend_mode
    @property
    def blend_mode(self) -> Optional[BlendMode]:
        return self.__blend_mode

    @blend_mode.setter
    def blend_mode(self, value: Optional[BlendMode]):
        self.__blend_mode = value
        self._set_enum_attr("blendMode", value, BlendMode)

    # color
    @property
    def color(self) -> Optional[ColorValue]:
        return self.__color

    @color.setter
    def color(self, value: Optional[ColorValue]):
        self.__color = value
        self._set_enum_attr("color", value, ColorEnums)

    # size
    @property
    def size(self) -> OptionalNumber:
        return self._get_attr("size", data_type="float")

    @size.setter
    def size(self, value: OptionalNumber):
        self._set_attr("size", value)

    # semantics_label
    @property
    def semantics_label(self) -> Optional[str]:
        return self._get_attr("semanticsLabel")

    @semantics_label.setter
    def semantics_label(self, value: Optional[str]):
        self._set_attr("semanticsLabel", value)

    # shadows
    @property
    def shadows(self) -> Union[BoxShadow, List[BoxShadow], None]:
        return self.__shadows

    @shadows.setter
    def shadows(self, value: Union[BoxShadow, List[BoxShadow], None]):
        self.__shadows = value

    # fill
    @property
    def fill(self) -> OptionalNumber:
        return self._get_attr("fill", data_type="float")

    @fill.setter
    def fill(self, value: OptionalNumber):
        self._set_attr("fill", value)

    # apply_text_scaling
    @property
    def apply_text_scaling(self) -> Optional[bool]:
        return self._get_attr("applyTextScaling", data_type="bool")

    @apply_text_scaling.setter
    def apply_text_scaling(self, value: Optional[bool]):
        self._set_attr("applyTextScaling", value)

    # grade
    @property
    def grade(self) -> OptionalNumber:
        return self._get_attr("grade", data_type="float")

    @grade.setter
    def grade(self, value: OptionalNumber):
        self._set_attr("grade", value)

    # weight
    @property
    def weight(self) -> OptionalNumber:
        return self._get_attr("weight", data_type="float")

    @weight.setter
    def weight(self, value: OptionalNumber):
        self._set_attr("weight", value)

    # optical_size
    @property
    def optical_size(self) -> OptionalNumber:
        return self._get_attr("opticalSize", data_type="float")

    @optical_size.setter
    def optical_size(self, value: OptionalNumber):
        self._set_attr("opticalSize", value)



from typing import Any, Optional, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    OffsetValue,
    OptionalControlEventCallable,
    PaddingValue,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
    UrlTarget,
)


class CupertinoListTile(ConstrainedControl):
    """
    An iOS-style list tile. The CupertinoListTile is a Cupertino equivalent of Material ListTile.

    Example:

    ```
    import flet as ft


    def main(page: ft.Page):
        def tile_clicked(e):
            print("Tile Clicked!")

        page.add(
            ft.CupertinoListTile(
                notched=True,
                additional_info=ft.Text("Wed Jan 25"),
                bgcolor_activated=ft.colors.AMBER_ACCENT,
                leading=ft.Icon(name=ft.cupertino_icons.GAME_CONTROLLER),
                title=ft.Text("CupertinoListTile not notched"),
                subtitle=ft.Text("Subtitle"),
                trailing=ft.Icon(name=ft.cupertino_icons.ALARM),
                on_click=tile_clicked,
            ),

        )

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/cupertinolisttile
    """

    def __init__(
        self,
        title: Optional[Control] = None,
        subtitle: Optional[Control] = None,
        leading: Optional[Control] = None,
        trailing: Optional[Control] = None,
        bgcolor: Optional[ColorValue] = None,
        bgcolor_activated: Optional[str] = None,
        padding: Optional[PaddingValue] = None,
        url: Optional[str] = None,
        url_target: Optional[UrlTarget] = None,
        toggle_inputs: Optional[bool] = None,
        additional_info: Optional[Control] = None,
        leading_size: OptionalNumber = None,
        leading_to_title: OptionalNumber = None,
        notched: Optional[bool] = None,
        on_click: OptionalControlEventCallable = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        self.leading = leading
        self.title = title
        self.subtitle = subtitle
        self.trailing = trailing
        self.bgcolor = bgcolor
        self.bgcolor_activated = bgcolor_activated
        self.url = url
        self.url_target = url_target
        self.toggle_inputs = toggle_inputs
        self.additional_info = additional_info
        self.leading_size = leading_size
        self.leading_to_title = leading_to_title
        self.padding = padding
        self.notched = notched
        self.on_click = on_click

    def _get_control_name(self):
        return "cupertinolisttile"

    def before_update(self):
        super().before_update()
        self._set_attr_json("padding", self.__padding)

    def _get_children(self):
        children = []
        if self.__leading:
            self.__leading._set_attr_internal("n", "leading")
            children.append(self.__leading)
        if self.__title:
            self.__title._set_attr_internal("n", "title")
            children.append(self.__title)
        if self.__subtitle:
            self.__subtitle._set_attr_internal("n", "subtitle")
            children.append(self.__subtitle)
        if self.__trailing:
            self.__trailing._set_attr_internal("n", "trailing")
            children.append(self.__trailing)
        if self.__additional_info:
            self.__additional_info._set_attr_internal("n", "additionalInfo")
            children.append(self.__additional_info)
        return children

    # leading
    @property
    def leading(self) -> Optional[Control]:
        return self.__leading

    @leading.setter
    def leading(self, value: Optional[Control]):
        self.__leading = value

    # leading_size
    @property
    def leading_size(self) -> float:
        return self._get_attr("leadingSize", data_type="float", def_value=30.0)

    @leading_size.setter
    def leading_size(self, value: OptionalNumber):
        self._set_attr("leadingSize", value)

    # leading_to_title
    @property
    def leading_to_title(self) -> float:
        return self._get_attr("leadingToTitle", data_type="float", def_value=12.0)

    @leading_to_title.setter
    def leading_to_title(self, value: OptionalNumber):
        self._set_attr("leadingToTitle", value)

    # title
    @property
    def title(self) -> Optional[Control]:
        return self.__title

    @title.setter
    def title(self, value: Optional[Control]):
        self.__title = value

    # subtitle
    @property
    def subtitle(self) -> Optional[Control]:
        return self.__subtitle

    @subtitle.setter
    def subtitle(self, value: Optional[Control]):
        self.__subtitle = value

    # trailing
    @property
    def trailing(self) -> Optional[Control]:
        return self.__trailing

    @trailing.setter
    def trailing(self, value: Optional[Control]):
        self.__trailing = value

    # additional_info
    @property
    def additional_info(self) -> Optional[Control]:
        return self.__additional_info

    @additional_info.setter
    def additional_info(self, value: Optional[Control]):
        self.__additional_info = value

    # padding
    @property
    def padding(self) -> Optional[PaddingValue]:
        return self.__padding

    @padding.setter
    def padding(self, value: Optional[PaddingValue]):
        self.__padding = value

    # notched
    @property
    def notched(self) -> bool:
        return self._get_attr("notched", data_type="bool", def_value=False)

    @notched.setter
    def notched(self, value: Optional[bool]):
        self._set_attr("notched", value)

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgcolor", value, ColorEnums)

    # bgcolor_activated
    @property
    def bgcolor_activated(self) -> Optional[str]:
        return self._get_attr("bgcolorActivated")

    @bgcolor_activated.setter
    def bgcolor_activated(self, value: Optional[str]):
        self._set_attr("bgcolorActivated", value)

    # url
    @property
    def url(self) -> Optional[str]:
        return self._get_attr("url")

    @url.setter
    def url(self, value: Optional[str]):
        self._set_attr("url", value)

    # url_target
    @property
    def url_target(self) -> Optional[UrlTarget]:
        return self.__url_target

    @url_target.setter
    def url_target(self, value: Optional[UrlTarget]):
        self.__url_target = value
        self._set_enum_attr("urlTarget", value, UrlTarget)

    # toggle_inputs
    @property
    def toggle_inputs(self) -> bool:
        return self._get_attr("toggleInputs", data_type="bool", def_value=False)

    @toggle_inputs.setter
    def toggle_inputs(self, value: Optional[bool]):
        self._set_attr("toggleInputs", value)

    # on_click
    @property
    def on_click(self) -> OptionalControlEventCallable:
        return self._get_event_handler("click")

    @on_click.setter
    def on_click(self, handler: OptionalControlEventCallable):
        self._add_event_handler("click", handler)
        self._set_attr("onclick", True if handler is not None else None)



from flet.app import app, app_async
from flet.core import (
    alignment,
    border,
    border_radius,
    dropdown,
    dropdownm2,
    margin,
    padding,
    painting,
    size,
)
from flet.core.adaptive_control import AdaptiveControl
from flet.core.alert_dialog import AlertDialog
from flet.core.alignment import Alignment, Axis
from flet.core.animated_switcher import AnimatedSwitcher, AnimatedSwitcherTransition
from flet.core.animation import Animation, AnimationCurve, AnimationStyle
from flet.core.app_bar import AppBar
from flet.core.audio import (
    Audio,
    AudioDurationChangeEvent,
    AudioPositionChangeEvent,
    AudioState,
    AudioStateChangeEvent,
)
from flet.core.audio_recorder import (
    AudioEncoder,
    AudioRecorder,
    AudioRecorderState,
    AudioRecorderStateChangeEvent,
)
from flet.core.auto_complete import (
    AutoComplete,
    AutoCompleteSelectEvent,
    AutoCompleteSuggestion,
)
from flet.core.autofill_group import (
    AutofillGroup,
    AutofillGroupDisposeAction,
    AutofillHint,
)
from flet.core.badge import Badge
from flet.core.banner import Banner
from flet.core.blur import Blur, BlurTileMode
from flet.core.border import Border, BorderSide, BorderSideStrokeAlign
from flet.core.border_radius import BorderRadius
from flet.core.bottom_app_bar import BottomAppBar
from flet.core.bottom_sheet import BottomSheet
from flet.core.box import (
    BoxConstraints,
    BoxDecoration,
    BoxShadow,
    BoxShape,
    ColorFilter,
    DecorationImage,
    FilterQuality,
    ShadowBlurStyle,
)
from flet.core.button import Button
from flet.core.buttons import (
    BeveledRectangleBorder,
    ButtonStyle,
    CircleBorder,
    ContinuousRectangleBorder,
    OutlinedBorder,
    RoundedRectangleBorder,
    StadiumBorder,
)
from flet.core.card import Card, CardVariant
from flet.core.charts.bar_chart import BarChart, BarChartEvent
from flet.core.charts.bar_chart_group import BarChartGroup
from flet.core.charts.bar_chart_rod import BarChartRod
from flet.core.charts.bar_chart_rod_stack_item import BarChartRodStackItem
from flet.core.charts.chart_axis import ChartAxis
from flet.core.charts.chart_axis_label import ChartAxisLabel
from flet.core.charts.chart_grid_lines import ChartGridLines
from flet.core.charts.chart_point_line import ChartPointLine
from flet.core.charts.chart_point_shape import (
    ChartCirclePoint,
    ChartCrossPoint,
    ChartPointShape,
    ChartSquarePoint,
)
from flet.core.charts.line_chart import LineChart, LineChartEvent, LineChartEventSpot
from flet.core.charts.line_chart_data import LineChartData
from flet.core.charts.line_chart_data_point import LineChartDataPoint
from flet.core.charts.pie_chart import PieChart, PieChartEvent
from flet.core.charts.pie_chart_section import PieChartSection
from flet.core.checkbox import Checkbox
from flet.core.chip import Chip
from flet.core.circle_avatar import CircleAvatar
from flet.core.colors import Colors
from flet.core.column import Column
from flet.core.container import Container, ContainerTapEvent
from flet.core.control import Control
from flet.core.control_event import ControlEvent
from flet.core.cupertino_action_sheet import CupertinoActionSheet
from flet.core.cupertino_action_sheet_action import CupertinoActionSheetAction
from flet.core.cupertino_activity_indicator import CupertinoActivityIndicator
from flet.core.cupertino_alert_dialog import CupertinoAlertDialog
from flet.core.cupertino_app_bar import CupertinoAppBar
from flet.core.cupertino_bottom_sheet import CupertinoBottomSheet
from flet.core.cupertino_button import CupertinoButton
from flet.core.cupertino_checkbox import CupertinoCheckbox
from flet.core.cupertino_colors import CupertinoColors
from flet.core.cupertino_context_menu import CupertinoContextMenu
from flet.core.cupertino_context_menu_action import CupertinoContextMenuAction
from flet.core.cupertino_date_picker import (
    CupertinoDatePicker,
    CupertinoDatePickerDateOrder,
    CupertinoDatePickerMode,
)
from flet.core.cupertino_dialog_action import CupertinoDialogAction
from flet.core.cupertino_filled_button import CupertinoFilledButton
from flet.core.cupertino_icons import CupertinoIcons
from flet.core.cupertino_list_tile import CupertinoListTile
from flet.core.cupertino_navigation_bar import CupertinoNavigationBar
from flet.core.cupertino_picker import CupertinoPicker
from flet.core.cupertino_radio import CupertinoRadio
from flet.core.cupertino_segmented_button import CupertinoSegmentedButton
from flet.core.cupertino_slider import CupertinoSlider
from flet.core.cupertino_sliding_segmented_button import CupertinoSlidingSegmentedButton
from flet.core.cupertino_switch import CupertinoSwitch
from flet.core.cupertino_textfield import CupertinoTextField, VisibilityMode
from flet.core.cupertino_timer_picker import (
    CupertinoTimerPicker,
    CupertinoTimerPickerMode,
)
from flet.core.datatable import (
    DataCell,
    DataColumn,
    DataColumnSortEvent,
    DataRow,
    DataTable,
)
from flet.core.date_picker import (
    DatePicker,
    DatePickerEntryMode,
    DatePickerEntryModeChangeEvent,
    DatePickerMode,
)
from flet.core.dismissible import (
    Dismissible,
    DismissibleDismissEvent,
    DismissibleUpdateEvent,
)
from flet.core.divider import Divider
from flet.core.drag_target import DragTarget, DragTargetEvent
from flet.core.draggable import Draggable
from flet.core.dropdown import Dropdown, DropdownOption
from flet.core.dropdownm2 import DropdownM2
from flet.core.elevated_button import ElevatedButton
from flet.core.exceptions import (
    FletException,
    FletUnimplementedPlatformEception,
    FletUnsupportedPlatformException,
)
from flet.core.expansion_panel import ExpansionPanel, ExpansionPanelList
from flet.core.expansion_tile import ExpansionTile, TileAffinity
from flet.core.file_picker import (
    FilePicker,
    FilePickerFileType,
    FilePickerResultEvent,
    FilePickerUploadEvent,
    FilePickerUploadFile,
)
from flet.core.filled_button import FilledButton
from flet.core.filled_tonal_button import FilledTonalButton
from flet.core.flashlight import Flashlight
from flet.core.flet_app import FletApp
from flet.core.floating_action_button import FloatingActionButton
from flet.core.form_field_control import InputBorder
from flet.core.geolocator import (
    Geolocator,
    GeolocatorActivityType,
    GeolocatorAndroidSettings,
    GeolocatorAppleSettings,
    GeolocatorPermissionStatus,
    GeolocatorPosition,
    GeolocatorPositionAccuracy,
    GeolocatorPositionChangeEvent,
    GeolocatorSettings,
    GeolocatorWebSettings,
)
from flet.core.gesture_detector import (
    DragEndEvent,
    DragStartEvent,
    DragUpdateEvent,
    GestureDetector,
    HoverEvent,
    LongPressEndEvent,
    LongPressStartEvent,
    MultiTapEvent,
    ScaleEndEvent,
    ScaleStartEvent,
    ScaleUpdateEvent,
    ScrollEvent,
    TapEvent,
)
from flet.core.gradients import (
    GradientTileMode,
    LinearGradient,
    RadialGradient,
    SweepGradient,
)
from flet.core.grid_view import GridView
from flet.core.haptic_feedback import HapticFeedback
from flet.core.icon import Icon
from flet.core.icon_button import IconButton
from flet.core.icons import Icons
from flet.core.image import Image
from flet.core.interactive_viewer import (
    InteractiveViewer,
    InteractiveViewerInteractionEndEvent,
    InteractiveViewerInteractionStartEvent,
    InteractiveViewerInteractionUpdateEvent,
)
from flet.core.list_tile import ListTile, ListTileStyle, ListTileTitleAlignment
from flet.core.list_view import ListView
from flet.core.lottie import Lottie
from flet.core.margin import Margin
from flet.core.markdown import (
    Markdown,
    MarkdownCodeTheme,
    MarkdownCustomCodeTheme,
    MarkdownExtensionSet,
    MarkdownStyleSheet,
)
from flet.core.menu_bar import MenuBar, MenuStyle
from flet.core.menu_item_button import MenuItemButton
from flet.core.navigation_bar import (
    NavigationBar,
    NavigationBarDestination,
    NavigationBarLabelBehavior,
)
from flet.core.navigation_drawer import (
    NavigationDrawer,
    NavigationDrawerDestination,
    NavigationDrawerPosition,
)
from flet.core.navigation_rail import (
    NavigationRail,
    NavigationRailDestination,
    NavigationRailLabelType,
)
from flet.core.outlined_button import OutlinedButton
from flet.core.padding import Padding
from flet.core.page import (
    AppLifecycleStateChangeEvent,
    BrowserContextMenu,
    KeyboardEvent,
    LoginEvent,
    Page,
    PageDisconnectedException,
    PageMediaData,
    RouteChangeEvent,
    ViewPopEvent,
    Window,
    WindowEvent,
    WindowResizeEvent,
    context,
)
from flet.core.pagelet import Pagelet
from flet.core.painting import (
    Paint,
    PaintingStyle,
    PaintLinearGradient,
    PaintRadialGradient,
    PaintSweepGradient,
)
from flet.core.permission_handler import (
    PermissionHandler,
    PermissionStatus,
    PermissionType,
)
from flet.core.placeholder import Placeholder
from flet.core.popup_menu_button import (
    PopupMenuButton,
    PopupMenuItem,
    PopupMenuPosition,
)
from flet.core.progress_bar import ProgressBar
from flet.core.progress_ring import ProgressRing
from flet.core.pubsub.pubsub_client import PubSubClient
from flet.core.pubsub.pubsub_hub import PubSubHub
from flet.core.querystring import QueryString
from flet.core.radio import Radio
from flet.core.radio_group import RadioGroup
from flet.core.range_slider import RangeSlider
from flet.core.ref import Ref
from flet.core.reorderable_draggable import ReorderableDraggable
from flet.core.reorderable_list_view import OnReorderEvent, ReorderableListView
from flet.core.responsive_row import ResponsiveRow
from flet.core.rive import Rive
from flet.core.row import Row
from flet.core.safe_area import SafeArea
from flet.core.scrollable_control import OnScrollEvent
from flet.core.search_bar import SearchBar
from flet.core.segmented_button import Segment, SegmentedButton
from flet.core.selection_area import SelectionArea
from flet.core.semantics import Semantics
from flet.core.semantics_service import Assertiveness, SemanticsService
from flet.core.shader_mask import ShaderMask
from flet.core.shake_detector import ShakeDetector
from flet.core.size import Size
from flet.core.slider import Slider, SliderInteraction
from flet.core.snack_bar import DismissDirection, SnackBar, SnackBarBehavior
from flet.core.stack import Stack, StackFit
from flet.core.submenu_button import SubmenuButton
from flet.core.switch import Switch
from flet.core.tabs import Tab, Tabs
from flet.core.template_route import TemplateRoute
from flet.core.text import Text, TextAffinity, TextSelection
from flet.core.text_button import TextButton
from flet.core.text_span import TextSpan
from flet.core.text_style import (
    TextBaseline,
    TextDecoration,
    TextDecorationStyle,
    TextOverflow,
    TextStyle,
    TextThemeStyle,
)
from flet.core.textfield import (
    InputFilter,
    KeyboardType,
    NumbersOnlyInputFilter,
    TextCapitalization,
    TextField,
    TextOnlyInputFilter,
)
from flet.core.theme import (
    AppBarTheme,
    BadgeTheme,
    BannerTheme,
    BottomAppBarTheme,
    BottomSheetTheme,
    ButtonTheme,
    CardTheme,
    CheckboxTheme,
    ChipTheme,
    ColorScheme,
    DataTableTheme,
    DatePickerTheme,
    DialogTheme,
    DividerTheme,
    ElevatedButtonTheme,
    ExpansionTileTheme,
    FilledButtonTheme,
    FloatingActionButtonTheme,
    IconButtonTheme,
    IconTheme,
    ListTileTheme,
    NavigationBarTheme,
    NavigationDrawerTheme,
    NavigationRailTheme,
    OutlinedButtonTheme,
    PageTransitionsTheme,
    PageTransitionTheme,
    PopupMenuTheme,
    ProgressIndicatorTheme,
    RadioTheme,
    ScrollbarTheme,
    SearchBarTheme,
    SearchViewTheme,
    SegmentedButtonTheme,
    SliderTheme,
    SnackBarTheme,
    SwitchTheme,
    SystemOverlayStyle,
    TabsTheme,
    TextButtonTheme,
    TextTheme,
    Theme,
    TimePickerTheme,
    TooltipTheme,
)
from flet.core.time_picker import (
    TimePicker,
    TimePickerEntryMode,
    TimePickerEntryModeChangeEvent,
)
from flet.core.tooltip import Tooltip, TooltipTriggerMode
from flet.core.transform import Offset, Rotate, Scale
from flet.core.transparent_pointer import TransparentPointer
from flet.core.types import (
    FLET_APP,
    FLET_APP_HIDDEN,
    FLET_APP_WEB,
    WEB_BROWSER,
    AppLifecycleState,
    AppView,
    BlendMode,
    BorderRadiusValue,
    Brightness,
    ClipBehavior,
    ColorEnums,
    ColorValue,
    ControlEventType,
    ControlState,
    ControlStateValue,
    CrossAxisAlignment,
    DateTimeValue,
    Duration,
    DurationValue,
    EventType,
    FloatingActionButtonLocation,
    FontWeight,
    IconEnums,
    IconValue,
    IconValueOrControl,
    ImageFit,
    ImageRepeat,
    LabelPosition,
    Locale,
    LocaleConfiguration,
    MainAxisAlignment,
    MarginValue,
    MouseCursor,
    NotchShape,
    Number,
    OffsetValue,
    OnFocusEvent,
    OptionalControlEventCallable,
    OptionalEventCallable,
    OptionalNumber,
    OptionalString,
    Orientation,
    PaddingValue,
    PagePlatform,
    PointerDeviceType,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
    ScrollMode,
    StrokeCap,
    StrokeJoin,
    SupportsStr,
    TabAlignment,
    TextAlign,
    ThemeMode,
    UrlTarget,
    VerticalAlignment,
    VisualDensity,
    WebRenderer,
    WindowEventType,
)
from flet.core.vertical_divider import VerticalDivider
from flet.core.video import (
    PlaylistMode,
    Video,
    VideoConfiguration,
    VideoMedia,
    VideoSubtitleConfiguration,
)
from flet.core.view import View
from flet.core.webview import (
    WebView,
    WebviewConsoleMessageEvent,
    WebviewJavaScriptEvent,
    WebviewLogLevelSeverity,
    WebviewRequestMethod,
    WebviewScrollEvent,
)
from flet.core.window_drag_area import WindowDragArea



from typing import Any, Optional, Union

from flet.core.badge import BadgeValue
from flet.core.charts.chart_point_line import ChartPointLine
from flet.core.charts.chart_point_shape import ChartPointShape
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.text_style import TextStyle
from flet.core.types import TextAlign


class LineChartDataPoint(Control):
    def __init__(
        self,
        x: OptionalNumber = None,
        y: OptionalNumber = None,
        selected: Optional[bool] = None,
        show_tooltip: Optional[bool] = None,
        tooltip: Optional[str] = None,
        tooltip_style: Optional[TextStyle] = None,
        tooltip_align: Optional[TextAlign] = None,
        point: Union[None, bool, ChartPointShape] = None,
        selected_point: Union[None, bool, ChartPointShape] = None,
        show_above_line: Optional[bool] = None,
        show_below_line: Optional[bool] = None,
        selected_below_line: Union[None, bool, ChartPointLine] = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        disabled: Optional[bool] = None,
        visible: Optional[bool] = None,
        badge: Optional[BadgeValue] = None,
        data: Any = None,
    ):

        Control.__init__(
            self,
            ref=ref,
            disabled=disabled,
            visible=visible,
            data=data,
            badge=badge,
        )

        self.x = x
        self.y = y
        self.selected = selected
        self.show_tooltip = show_tooltip
        self.tooltip = tooltip
        self.tooltip_align = tooltip_align
        self.tooltip_style = tooltip_style
        self.point = point
        self.selected_point = selected_point
        self.show_above_line = show_above_line
        self.show_below_line = show_below_line
        self.selected_below_line = selected_below_line

    def _get_control_name(self):
        return "p"

    def before_update(self):
        super().before_update()
        self._set_attr_json("tooltipStyle", self.__tooltip_style)
        self._set_attr_json("point", self.__point)
        self._set_attr_json("selectedPoint", self.__selected_point)
        self._set_attr_json("selectedBelowLine", self.__selected_below_line)

    def _get_children(self):
        children = []
        return children

    # x
    @property
    def x(self) -> float:
        return self._get_attr("x", data_type="float", def_value=0.0)

    @x.setter
    def x(self, value: OptionalNumber):
        self._set_attr("x", value)

    # y
    @property
    def y(self) -> float:
        return self._get_attr("y", data_type="float", def_value=0.0)

    @y.setter
    def y(self, value: OptionalNumber):
        self._set_attr("y", value)

    # selected
    @property
    def selected(self) -> bool:
        return self._get_attr("selected", data_type="bool", def_value=False)

    @selected.setter
    def selected(self, value: Optional[bool]):
        self._set_attr("selected", value)

    # show_tooltip
    @property
    def show_tooltip(self) -> bool:
        return self._get_attr("showTooltip", data_type="bool", def_value=True)

    @show_tooltip.setter
    def show_tooltip(self, value: Optional[bool]):
        self._set_attr("showTooltip", value)

    # tooltip
    @property
    def tooltip(self) -> Optional[str]:
        return self._get_attr("tooltip")

    @tooltip.setter
    def tooltip(self, value: Optional[str]):
        self._set_attr("tooltip", value)

    # tooltip_align
    @property
    def tooltip_align(self) -> Optional[TextAlign]:
        return self.__tooltip_align

    @tooltip_align.setter
    def tooltip_align(self, value: Optional[TextAlign]):
        self.__tooltip_align = value
        self._set_attr(
            "tooltipAlign", value.value if isinstance(value, TextAlign) else value
        )

    # tooltip_style
    @property
    def tooltip_style(self):
        return self.__tooltip_style

    @tooltip_style.setter
    def tooltip_style(self, value: Optional[TextStyle]):
        self.__tooltip_style = value

    # point
    @property
    def point(self):
        return self.__point

    @point.setter
    def point(self, value: Union[None, bool, ChartPointShape]):
        self.__point = value

    # selected_point
    @property
    def selected_point(self):
        return self.__selected_point

    @selected_point.setter
    def selected_point(self, value: Union[None, bool, ChartPointShape]):
        self.__selected_point = value

    # show_above_line
    @property
    def show_above_line(self) -> bool:
        return self._get_attr("showAboveLine", data_type="bool", def_value=True)

    @show_above_line.setter
    def show_above_line(self, value: Optional[bool]):
        self._set_attr("showAboveLine", value)

    # show_below_line
    @property
    def show_below_line(self) -> bool:
        return self._get_attr("showBelowLine", data_type="bool", def_value=True)

    @show_below_line.setter
    def show_below_line(self, value: Optional[bool]):
        self._set_attr("showBelowLine", value)

    # selected_below_line
    @property
    def selected_below_line(self) -> Union[None, bool, ChartPointLine]:
        return self.__selected_below_line

    @selected_below_line.setter
    def selected_below_line(self, value: Union[None, bool, ChartPointLine]):
        self.__selected_below_line = value



from typing import Any, Optional, Union

from flet.core.alignment import Alignment
from flet.core.badge import BadgeValue
from flet.core.control import Control, OptionalNumber
from flet.core.cupertino_button import CupertinoButton
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    BorderRadiusValue,
    ColorValue,
    IconValue,
    OptionalControlEventCallable,
    PaddingValue,
    ResponsiveNumber,
    UrlTarget,
)


class CupertinoFilledButton(CupertinoButton):
    """
    An iOS-style button filled with default background color.

    Example:
    ```
    import flet as ft


    def main(page: ft.Page):
        page.add(
            ft.CupertinoFilledButton(text="OK"),
        )

    ft.app(target=main)
    ```
    -----

    Online docs: https://flet.dev/docs/controls/cupertinofilledbutton
    """

    def __init__(
        self,
        text: Optional[str] = None,
        icon: Optional[IconValue] = None,
        icon_color: Optional[ColorValue] = None,
        content: Optional[Control] = None,
        disabled_bgcolor: Optional[ColorValue] = None,
        opacity_on_click: OptionalNumber = None,
        min_size: OptionalNumber = None,
        padding: Optional[PaddingValue] = None,
        alignment: Optional[Alignment] = None,
        border_radius: Optional[BorderRadiusValue] = None,
        url: Optional[str] = None,
        autofocus: Optional[bool] = None,
        focus_color: Optional[bool] = None,
        url_target: Optional[UrlTarget] = None,
        on_click: OptionalControlEventCallable = None,
        on_long_press: OptionalControlEventCallable = None,
        on_focus: OptionalControlEventCallable = None,
        on_blur: OptionalControlEventCallable = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        CupertinoButton.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
            #
            # Specific
            #
            color="onPrimary",
            bgcolor="primary",
            disabled_bgcolor=disabled_bgcolor,
            text=text,
            icon=icon,
            icon_color=icon_color,
            content=content,
            url=url,
            url_target=url_target,
            on_click=on_click,
            border_radius=border_radius,
            min_size=min_size,
            opacity_on_click=opacity_on_click,
            padding=padding,
            alignment=alignment,
            autofocus=autofocus,
            focus_color=focus_color,
            on_long_press=on_long_press,
            on_focus=on_focus,
            on_blur=on_blur,
        )



from typing import Any, Optional, Union

from flet.core.adaptive_control import AdaptiveControl
from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import Control, OptionalNumber
from flet.core.event_handler import EventHandler
from flet.core.ref import Ref
from flet.core.text_style import TextStyle
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    ControlStateValue,
    IconValue,
    LabelPosition,
    MouseCursor,
    OffsetValue,
    OnFocusEvent,
    OptionalControlEventCallable,
    OptionalEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
)


class Switch(ConstrainedControl, AdaptiveControl):
    """
    A toggle represents a physical switch that allows someone to choose between two mutually exclusive options.

    or example, "On/Off", "Show/Hide". Choosing an option should produce an immediate result.

    Example:
    ```
    import flet as ft

    def main(page: ft.Page):
        def theme_changed(e):
            page.theme_mode = (
                ft.ThemeMode.DARK
                if page.theme_mode == ft.ThemeMode.LIGHT
                else ft.ThemeMode.LIGHT
            )
            c.label = (
                "Light theme" if page.theme_mode == ft.ThemeMode.LIGHT else "Dark theme"
            )
            page.update()

        page.theme_mode = ft.ThemeMode.LIGHT
        c = ft.Switch(label="Light theme", on_change=theme_changed)
        page.add(c)

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/switch
    """

    def __init__(
        self,
        label: Optional[Union[str, Control]] = None,
        label_position: Optional[LabelPosition] = None,
        label_style: Optional[TextStyle] = None,
        value: Optional[bool] = None,
        autofocus: Optional[bool] = None,
        active_color: Optional[ColorValue] = None,
        active_track_color: Optional[ColorValue] = None,
        focus_color: Optional[ColorValue] = None,
        inactive_thumb_color: Optional[ColorValue] = None,
        inactive_track_color: Optional[ColorValue] = None,
        thumb_color: ControlStateValue[ColorValue] = None,
        thumb_icon: ControlStateValue[IconValue] = None,
        track_color: ControlStateValue[ColorValue] = None,
        adaptive: Optional[bool] = None,
        hover_color: Optional[ColorValue] = None,
        splash_radius: OptionalNumber = None,
        overlay_color: ControlStateValue[ColorValue] = None,
        track_outline_color: ControlStateValue[ColorValue] = None,
        track_outline_width: ControlStateValue[OptionalNumber] = None,
        mouse_cursor: Optional[MouseCursor] = None,
        on_change: OptionalControlEventCallable = None,
        on_focus: OptionalEventCallable[OnFocusEvent] = None,
        on_blur: OptionalControlEventCallable = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        AdaptiveControl.__init__(self, adaptive=adaptive)

        self.__on_focus = EventHandler(lambda e: OnFocusEvent(e))
        self._add_event_handler("focus", self.__on_focus.get_handler())

        self.value = value
        self.label = label
        self.label_style = label_style
        self.label_position = label_position
        self.autofocus = autofocus
        self.active_color = active_color
        self.active_track_color = active_track_color
        self.focus_color = focus_color
        self.inactive_thumb_color = inactive_thumb_color
        self.inactive_track_color = inactive_track_color
        self.thumb_color = thumb_color
        self.thumb_icon = thumb_icon
        self.track_color = track_color
        self.on_change = on_change
        self.on_focus = on_focus
        self.on_blur = on_blur
        self.hover_color = hover_color
        self.splash_radius = splash_radius
        self.overlay_color = overlay_color
        self.track_outline_color = track_outline_color
        self.track_outline_width = track_outline_width
        self.mouse_cursor = mouse_cursor

    def _get_control_name(self):
        return "switch"

    def before_update(self):
        super().before_update()
        self._set_attr_json("thumbColor", self.__thumb_color, wrap_attr_dict=True)
        self._set_attr_json("overlayColor", self.__overlay_color, wrap_attr_dict=True)
        self._set_attr_json(
            "trackOutlineColor", self.__track_outline_color, wrap_attr_dict=True
        )
        self._set_attr_json(
            "trackOutlineWidth", self.__track_outline_width, wrap_attr_dict=True
        )
        self._set_attr_json("thumbIcon", self.__thumb_icon, wrap_attr_dict=True)
        self._set_attr_json("trackColor", self.__track_color, wrap_attr_dict=True)
        self._set_attr_json("labelStyle", self.__label_style)

    def _get_children(self):
        return [self.__label] if isinstance(self.__label, Control) else []

    # value
    @property
    def value(self) -> bool:
        return self._get_attr("value", data_type="bool", def_value=False)

    @value.setter
    def value(self, value: Optional[bool]):
        self._set_attr("value", value)

    # label
    @property
    def label(self) -> Optional[Union[str, Control]]:
        return self.__label

    @label.setter
    def label(self, value: Optional[Union[str, Control]]):
        self.__label = value
        if not isinstance(value, Control):
            self._set_attr("label", value)

    # hover_color
    @property
    def hover_color(self) -> Optional[ColorValue]:
        return self.__hover_color

    @hover_color.setter
    def hover_color(self, value: Optional[ColorValue]):
        self.__hover_color = value
        self._set_enum_attr("hoverColor", value, ColorEnums)

    # track_outline_color
    @property
    def track_outline_color(self) -> ControlStateValue[ColorValue]:
        return self.__track_outline_color

    @track_outline_color.setter
    def track_outline_color(self, value: ControlStateValue[ColorValue]):
        self.__track_outline_color = value

    # track_outline_width
    @property
    def track_outline_width(self) -> ControlStateValue[OptionalNumber]:
        return self.__track_outline_width

    @track_outline_width.setter
    def track_outline_width(self, value: ControlStateValue[OptionalNumber]):
        self.__track_outline_width = value

    # overlay_color
    @property
    def overlay_color(self) -> ControlStateValue[str]:
        return self.__overlay_color

    @overlay_color.setter
    def overlay_color(self, value: ControlStateValue[str]):
        self.__overlay_color = value

    # splash_radius
    @property
    def splash_radius(self) -> OptionalNumber:
        return self._get_attr("splashRadius", data_type="float")

    @splash_radius.setter
    def splash_radius(self, value: OptionalNumber):
        assert value is None or value >= 0, "splash_radius cannot be negative"
        self._set_attr("splashRadius", value)

    # label_style
    @property
    def label_style(self) -> Optional[TextStyle]:
        return self.__label_style

    @label_style.setter
    def label_style(self, value: Optional[TextStyle]):
        self.__label_style = value

    # label_position
    @property
    def label_position(self) -> Optional[LabelPosition]:
        return self.__label_position

    @label_position.setter
    def label_position(self, value: Optional[LabelPosition]):
        self.__label_position = value
        self._set_enum_attr("labelPosition", value, LabelPosition)

    # mouse_cursor
    @property
    def mouse_cursor(self) -> Optional[MouseCursor]:
        return self.__mouse_cursor

    @mouse_cursor.setter
    def mouse_cursor(self, value: Optional[MouseCursor]):
        self.__mouse_cursor = value
        self._set_enum_attr("mouseCursor", value, MouseCursor)

    # autofocus
    @property
    def autofocus(self) -> bool:
        return self._get_attr("autofocus", data_type="bool", def_value=False)

    @autofocus.setter
    def autofocus(self, value: Optional[bool]):
        self._set_attr("autofocus", value)

    # active_color
    @property
    def active_color(self) -> Optional[ColorValue]:
        return self.__active_color

    @active_color.setter
    def active_color(self, value: Optional[ColorValue]):
        self.__active_color = value
        self._set_enum_attr("activeColor", value, ColorEnums)

    # active_track_color
    @property
    def active_track_color(self) -> Optional[ColorValue]:
        return self.__active_track_color

    @active_track_color.setter
    def active_track_color(self, value: Optional[ColorValue]):
        self.__active_track_color = value
        self._set_enum_attr("activeTrackColor", value, ColorEnums)

    # focus_color
    @property
    def focus_color(self) -> Optional[ColorValue]:
        return self.__focus_color

    @focus_color.setter
    def focus_color(self, value: Optional[ColorValue]):
        self.__focus_color = value
        self._set_enum_attr("focusColor", value, ColorEnums)

    # inactive_thumb_color
    @property
    def inactive_thumb_color(self) -> Optional[ColorValue]:
        return self.__inactive_thumb_color

    @inactive_thumb_color.setter
    def inactive_thumb_color(self, value: Optional[ColorValue]):
        self.__inactive_thumb_color = value
        self._set_enum_attr("inactiveThumbColor", value, ColorEnums)

    # inactive_track_color
    @property
    def inactive_track_color(self) -> Optional[ColorValue]:
        return self.__inactive_track_color

    @inactive_track_color.setter
    def inactive_track_color(self, value: Optional[ColorValue]):
        self.__inactive_track_color = value
        self._set_enum_attr("inactiveTrackColor", value, ColorEnums)

    # thumb_color
    @property
    def thumb_color(self) -> ControlStateValue[str]:
        return self.__thumb_color

    @thumb_color.setter
    def thumb_color(self, value: ControlStateValue[str]):
        self.__thumb_color = value

    # thumb_icon
    @property
    def thumb_icon(self) -> ControlStateValue[IconValue]:
        return self.__thumb_icon

    @thumb_icon.setter
    def thumb_icon(self, value: ControlStateValue[IconValue]):
        self.__thumb_icon = value

    # track_color
    @property
    def track_color(self) -> ControlStateValue[str]:
        return self.__track_color

    @track_color.setter
    def track_color(self, value: ControlStateValue[str]):
        self.__track_color = value

    # on_change
    @property
    def on_change(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change")

    @on_change.setter
    def on_change(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change", handler)

    # on_focus
    @property
    def on_focus(self) -> OptionalEventCallable[OnFocusEvent]:
        return self.__on_focus.handler

    @on_focus.setter
    def on_focus(self, handler: OptionalEventCallable[OnFocusEvent]):
        self.__on_focus.handler = handler

    # on_blur
    @property
    def on_blur(self) -> OptionalControlEventCallable:
        return self._get_event_handler("blur")

    @on_blur.setter
    def on_blur(self, handler: OptionalControlEventCallable):
        self._add_event_handler("blur", handler)



from typing import Any, Optional

from flet.core.alignment import Alignment
from flet.core.map.map_layer import MapLayer
from flet.core.ref import Ref
from flet.core.types import ColorEnums, ColorValue, OptionalControlEventCallable


class SimpleAttribution(MapLayer):
    """
    A simple attribution layer displayed on the Map.

    -----

    Online docs: https://flet.dev/docs/controls/mapsimpleattribution
    """

    def __init__(
        self,
        text: str,
        alignment: Optional[Alignment] = None,
        bgcolor: Optional[ColorValue] = None,
        on_click: OptionalControlEventCallable = None,
        #
        # MapLayer
        #
        ref: Optional[Ref] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):

        MapLayer.__init__(
            self,
            ref=ref,
            visible=visible,
            data=data,
        )

        self.text = text
        self.alignment = alignment
        self.bgcolor = bgcolor
        self.on_click = on_click

    def _get_control_name(self):
        return "map_simple_attribution"

    def before_update(self):
        super().before_update()
        self._set_attr_json("alignment", self.__alignment)

    # alignment
    @property
    def alignment(self) -> Optional[Alignment]:
        return self.__alignment

    @alignment.setter
    def alignment(self, value: Optional[Alignment]):
        self.__alignment = value

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgcolor", value, ColorEnums)

    # text
    @property
    def text(self) -> str:
        return self._get_attr("text")

    @text.setter
    def text(self, value: str):
        self._set_attr("text", value)

    # on_click
    @property
    def on_change(self) -> OptionalControlEventCallable:
        return self._get_event_handler("click")

    @on_change.setter
    def on_change(self, handler: OptionalControlEventCallable):
        self._add_event_handler("click", handler)



from typing import Any, List, Optional, Union

from flet.core.box import BoxShadow
from flet.core.charts.chart_point_line import ChartPointLine
from flet.core.charts.chart_point_shape import ChartPointShape
from flet.core.charts.line_chart_data_point import LineChartDataPoint
from flet.core.control import Control, OptionalNumber
from flet.core.gradients import Gradient
from flet.core.ref import Ref
from flet.core.types import ColorEnums, ColorValue


class LineChartData(Control):
    def __init__(
        self,
        data_points: Optional[List[LineChartDataPoint]] = None,
        curved: Optional[bool] = None,
        color: Optional[ColorValue] = None,
        gradient: Optional[Gradient] = None,
        stroke_width: OptionalNumber = None,
        stroke_cap_round: Optional[bool] = None,
        prevent_curve_over_shooting: Optional[bool] = None,
        prevent_curve_over_shooting_threshold: OptionalNumber = None,
        dash_pattern: Optional[List[int]] = None,
        shadow: Optional[BoxShadow] = None,
        above_line_bgcolor: Optional[ColorValue] = None,
        above_line_gradient: Optional[Gradient] = None,
        above_line_cutoff_y: OptionalNumber = None,
        above_line: Optional[ChartPointLine] = None,
        below_line_bgcolor: Optional[ColorValue] = None,
        below_line_gradient: Optional[Gradient] = None,
        below_line_cutoff_y: OptionalNumber = None,
        below_line: Optional[ChartPointLine] = None,
        selected_below_line: Union[None, bool, ChartPointLine] = None,
        point: Union[None, bool, ChartPointShape] = None,
        selected_point: Union[None, bool, ChartPointShape] = None,
        #
        # Control
        #
        ref: Optional[Ref] = None,
        disabled: Optional[bool] = None,
        visible: Optional[bool] = None,
        data: Any = None,
    ):

        Control.__init__(
            self,
            ref=ref,
            disabled=disabled,
            visible=visible,
            data=data,
        )

        self.data_points = data_points
        self.curved = curved
        self.color = color
        self.gradient = gradient
        self.stroke_width = stroke_width
        self.stroke_cap_round = stroke_cap_round
        self.prevent_curve_over_shooting = prevent_curve_over_shooting
        self.prevent_curve_over_shooting_threshold = (
            prevent_curve_over_shooting_threshold
        )
        self.shadow = shadow
        self.dash_pattern = dash_pattern
        self.above_line_bgcolor = above_line_bgcolor
        self.above_line_gradient = above_line_gradient
        self.above_line_cutoff_y = above_line_cutoff_y
        self.above_line = above_line
        self.below_line_bgcolor = below_line_bgcolor
        self.below_line_gradient = below_line_gradient
        self.below_line_cutoff_y = below_line_cutoff_y
        self.below_line = below_line
        self.selected_below_line = selected_below_line
        self.point = point
        self.selected_point = selected_point

    def _get_control_name(self):
        return "data"

    def before_update(self):
        super().before_update()
        self._set_attr_json("gradient", self.__gradient)
        self._set_attr_json("shadow", self.__shadow)
        self._set_attr_json("point", self.__point)
        self._set_attr_json("selectedPoint", self.__selected_point)
        self._set_attr_json("dashPattern", self.__dash_pattern)
        self._set_attr_json("aboveLineGradient", self.__above_line_gradient)
        self._set_attr_json("belowLineGradient", self.__below_line_gradient)
        self._set_attr_json("aboveLine", self.__above_line)
        self._set_attr_json("belowLine", self.__below_line)
        self._set_attr_json("selectedBelowLine", self.__selected_below_line)

    def _get_children(self):
        return self.__data_points

    # data_points
    @property
    def data_points(self):
        return self.__data_points

    @data_points.setter
    def data_points(self, value):
        self.__data_points = value if value is not None else []

    # stroke_width
    @property
    def stroke_width(self) -> float:
        return self._get_attr("strokeWidth", data_type="float", def_value=1.0)

    @stroke_width.setter
    def stroke_width(self, value: OptionalNumber):
        self._set_attr("strokeWidth", value)

    # curved
    @property
    def curved(self) -> bool:
        return self._get_attr("curved", data_type="bool", def_value=False)

    @curved.setter
    def curved(self, value: Optional[bool]):
        self._set_attr("curved", value)

    # color
    @property
    def color(self) -> Optional[ColorValue]:
        return self.__color

    @color.setter
    def color(self, value: Optional[ColorValue]):
        self.__color = value
        self._set_enum_attr("color", value, ColorEnums)

    # gradient
    @property
    def gradient(self) -> Optional[Gradient]:
        return self.__gradient

    @gradient.setter
    def gradient(self, value: Optional[Gradient]):
        self.__gradient = value

    # stroke_cap_round
    @property
    def stroke_cap_round(self) -> bool:
        return self._get_attr("strokeCapRound", data_type="bool", def_value=False)

    @stroke_cap_round.setter
    def stroke_cap_round(self, value: Optional[bool]):
        self._set_attr("strokeCapRound", value)

    # prevent_curve_over_shooting
    @property
    def prevent_curve_over_shooting(self) -> bool:
        return self._get_attr(
            "preventCurveOverShooting", data_type="bool", def_value=False
        )

    @prevent_curve_over_shooting.setter
    def prevent_curve_over_shooting(self, value: Optional[bool]):
        self._set_attr("preventCurveOverShooting", value)

    # prevent_curve_over_shooting_threshold
    @property
    def prevent_curve_over_shooting_threshold(self) -> OptionalNumber:
        return self._get_attr("preventCurveOverShootingThreshold", data_type="float")

    @prevent_curve_over_shooting_threshold.setter
    def prevent_curve_over_shooting_threshold(self, value: OptionalNumber):
        self._set_attr("preventCurveOverShootingThreshold", value)

    # dash_pattern
    @property
    def dash_pattern(self):
        return self.__dash_pattern

    @dash_pattern.setter
    def dash_pattern(self, value: Optional[List[int]]):
        self.__dash_pattern = value

    # shadow
    @property
    def shadow(self):
        return self.__shadow

    @shadow.setter
    def shadow(self, value: Optional[BoxShadow]):
        self.__shadow = value

    # point
    @property
    def point(self):
        return self.__point

    @point.setter
    def point(self, value: Union[None, bool, ChartPointShape]):
        self.__point = value

    # selected_point
    @property
    def selected_point(self):
        return self.__selected_point

    @selected_point.setter
    def selected_point(self, value: Union[None, bool, ChartPointShape]):
        self.__selected_point = value

    # above_line_bgcolor
    @property
    def above_line_bgcolor(self) -> Optional[str]:
        return self.__above_line_bgcolor

    @above_line_bgcolor.setter
    def above_line_bgcolor(self, value: Optional[str]):
        self.__above_line_bgcolor = value
        self._set_enum_attr("aboveLineBgcolor", value, ColorEnums)

    # above_line_gradient
    @property
    def above_line_gradient(self) -> Optional[Gradient]:
        return self.__above_line_gradient

    @above_line_gradient.setter
    def above_line_gradient(self, value: Optional[Gradient]):
        self.__above_line_gradient = value

    # above_line_cutoff_y
    @property
    def above_line_cutoff_y(self) -> OptionalNumber:
        return self._get_attr("aboveLineCutoffY", data_type="float")

    @above_line_cutoff_y.setter
    def above_line_cutoff_y(self, value: OptionalNumber):
        self._set_attr("aboveLineCutoffY", value)

    # above_line
    @property
    def above_line(self) -> Optional[ChartPointLine]:
        return self.__above_line

    @above_line.setter
    def above_line(self, value: Optional[ChartPointLine]):
        self.__above_line = value

    # below_line_bgcolor
    @property
    def below_line_bgcolor(self) -> Optional[str]:
        return self.__below_line_bgcolor

    @below_line_bgcolor.setter
    def below_line_bgcolor(self, value: Optional[str]):
        self.__below_line_bgcolor = value
        self._set_enum_attr("belowLineBgcolor", value, ColorEnums)

    # below_line_gradient
    @property
    def below_line_gradient(self) -> Optional[Gradient]:
        return self.__below_line_gradient

    @below_line_gradient.setter
    def below_line_gradient(self, value: Optional[Gradient]):
        self.__below_line_gradient = value

    # below_line_cutoff_y
    @property
    def below_line_cutoff_y(self) -> OptionalNumber:
        return self._get_attr("belowLineCutoffY", data_type="float")

    @below_line_cutoff_y.setter
    def below_line_cutoff_y(self, value: OptionalNumber):
        self._set_attr("belowLineCutoffY", value)

    # below_line
    @property
    def below_line(self) -> Optional[ChartPointLine]:
        return self.__below_line

    @below_line.setter
    def below_line(self, value: Optional[ChartPointLine]):
        self.__below_line = value

    # selected_below_line
    @property
    def selected_below_line(self) -> Union[None, bool, ChartPointLine]:
        return self.__selected_below_line

    @selected_below_line.setter
    def selected_below_line(self, value: Union[None, bool, ChartPointLine]):
        self.__selected_below_line = value



from typing import Any, Optional, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    OffsetValue,
    OptionalControlEventCallable,
    RotateValue,
    ScaleValue,
)


class TransparentPointer(ConstrainedControl):
    def __init__(
        self,
        content: Optional[Control] = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        self.content = content

    def _get_control_name(self):
        return "transparentpointer"

    def _get_children(self):
        if self.__content is not None:
            self.__content._set_attr_internal("n", "content")
            return [self.__content]
        return []

    # content
    @property
    def content(self) -> Optional[Control]:
        return self.__content

    @content.setter
    def content(self, value: Optional[Control]):
        self.__content = value



import time
from typing import Any, Dict, List, Optional, Sequence, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.border import BorderSide
from flet.core.box import BoxConstraints
from flet.core.buttons import OutlinedBorder
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import Control
from flet.core.ref import Ref
from flet.core.text_style import TextStyle
from flet.core.textfield import KeyboardType, TextCapitalization
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    ControlState,
    ControlStateValue,
    Number,
    OffsetValue,
    OptionalControlEventCallable,
    OptionalNumber,
    PaddingValue,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
)


class SearchBar(ConstrainedControl):
    """
    Manages a "search view" route that allows the user to select one of the suggested completions for a search query.

    -----

    Online docs: https://flet.dev/docs/controls/searchbar
    """

    def __init__(
        self,
        controls: Optional[Sequence[Control]] = None,
        value: Optional[str] = None,
        bar_leading: Optional[Control] = None,
        bar_trailing: Optional[List[Control]] = None,
        bar_hint_text: Optional[str] = None,
        bar_bgcolor: ControlStateValue[ColorValue] = None,
        bar_overlay_color: ControlStateValue[ColorValue] = None,
        bar_shadow_color: ControlStateValue[ColorValue] = None,
        bar_surface_tint_color: ControlStateValue[ColorValue] = None,
        bar_elevation: ControlStateValue[OptionalNumber] = None,
        bar_border_side: ControlStateValue[BorderSide] = None,
        bar_shape: ControlStateValue[OutlinedBorder] = None,
        bar_text_style: ControlStateValue[TextStyle] = None,
        bar_hint_text_style: ControlStateValue[TextStyle] = None,
        bar_padding: ControlStateValue[PaddingValue] = None,
        bar_scroll_padding: Optional[PaddingValue] = None,
        view_leading: Optional[Control] = None,
        view_trailing: Optional[List[Control]] = None,
        view_elevation: OptionalNumber = None,
        view_bgcolor: Optional[ColorValue] = None,
        view_hint_text: Optional[str] = None,
        view_side: Optional[BorderSide] = None,
        view_shape: Optional[OutlinedBorder] = None,
        view_header_text_style: Optional[TextStyle] = None,
        view_hint_text_style: Optional[TextStyle] = None,
        view_size_constraints: Optional[BoxConstraints] = None,
        view_header_height: OptionalNumber = None,
        divider_color: Optional[ColorValue] = None,
        capitalization: Optional[TextCapitalization] = None,
        full_screen: Optional[bool] = None,
        keyboard_type: Optional[KeyboardType] = None,
        view_surface_tint_color: Optional[ColorValue] = None,
        autofocus: Optional[bool] = None,
        on_tap: OptionalControlEventCallable = None,
        on_tap_outside_bar: OptionalControlEventCallable = None,
        on_submit: OptionalControlEventCallable = None,
        on_change: OptionalControlEventCallable = None,
        on_focus: OptionalControlEventCallable = None,
        on_blur: OptionalControlEventCallable = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        self.value = value
        self.controls = controls
        self.bar_leading = bar_leading
        self.bar_trailing = bar_trailing
        self.bar_hint_text = bar_hint_text
        self.bar_bgcolor = bar_bgcolor
        self.bar_overlay_color = bar_overlay_color
        self.view_leading = view_leading
        self.view_trailing = view_trailing
        self.view_elevation = view_elevation
        self.view_bgcolor = view_bgcolor
        self.view_hint_text = view_hint_text
        self.view_side = view_side
        self.view_shape = view_shape
        self.view_header_text_style = view_header_text_style
        self.view_hint_text_style = view_hint_text_style
        self.divider_color = divider_color
        self.full_screen = full_screen
        self.capitalization = capitalization
        self.on_focus = on_focus
        self.on_blur = on_blur
        self.on_tap = on_tap
        self.on_submit = on_submit
        self.on_change = on_change
        self.on_tap_outside_bar = on_tap_outside_bar
        self.keyboard_type = keyboard_type
        self.view_surface_tint_color = view_surface_tint_color
        self.autofocus = autofocus
        self.view_header_height = view_header_height
        self.view_size_constraints = view_size_constraints
        self.bar_surface_tint_color = bar_surface_tint_color
        self.bar_elevation = bar_elevation
        self.bar_border_side = bar_border_side
        self.bar_shape = bar_shape
        self.bar_text_style = bar_text_style
        self.bar_hint_text_style = bar_hint_text_style
        self.bar_padding = bar_padding
        self.bar_scroll_padding = bar_scroll_padding
        self.bar_shadow_color = bar_shadow_color

    def _get_control_name(self):
        return "searchbar"

    def __contains__(self, item):
        return item in self.__controls

    def before_update(self):
        super().before_update()
        self._set_attr_json("barBgcolor", self.__bar_bgcolor, wrap_attr_dict=True)
        self._set_attr_json(
            "barOverlayColor", self.__bar_overlay_color, wrap_attr_dict=True
        )
        self._set_attr_json(
            "barHintTextStyle", self.__bar_hint_text_style, wrap_attr_dict=True
        )
        self._set_attr_json(
            "barSurfaceTintColor", self.__bar_surface_tint_color, wrap_attr_dict=True
        )
        self._set_attr_json("barElevation", self.__bar_elevation, wrap_attr_dict=True)
        self._set_attr_json(
            "barBorderSide", self.__bar_border_side, wrap_attr_dict=True
        )
        self._set_attr_json("barShape", self.__bar_shape, wrap_attr_dict=True)
        self._set_attr_json("barTextStyle", self.__bar_text_style, wrap_attr_dict=True)
        self._set_attr_json("barPadding", self.__bar_padding, wrap_attr_dict=True)
        self._set_attr_json(
            "barShadowColor", self.__bar_shadow_color, wrap_attr_dict=True
        )
        self._set_attr_json("viewShape", self.__view_shape)
        self._set_attr_json("viewHeaderTextStyle", self.__view_header_text_style)
        self._set_attr_json("viewHintTextStyle", self.__view_hint_text_style)
        self._set_attr_json("viewSide", self.__view_side)
        self._set_attr_json("viewSizeConstraints", self.__view_size_constraints)
        self._set_attr_json("barScrollPadding", self.__bar_scroll_padding)

    def _get_children(self):
        children = []
        if self.__bar_leading:
            self.__bar_leading._set_attr_internal("n", "barLeading")
            children.append(self.__bar_leading)
        if self.__view_leading:
            self.__view_leading._set_attr_internal("n", "viewLeading")
            children.append(self.__view_leading)
        for i in self.__bar_trailing:
            i._set_attr_internal("n", "barTrailing")
            children.append(i)
        for i in self.__view_trailing:
            i._set_attr_internal("n", "viewTrailing")
            children.append(i)
        for i in self.__controls:
            i._set_attr_internal("n", "controls")
            children.append(i)
        return children

    # Public methods
    def focus(self):
        self._set_attr_json("focus", str(time.time()))
        self.update()

    def blur(self):
        self._set_attr_json("blur", str(time.time()))
        self.update()

    def open_view(self):
        m = {
            "n": "openView",
            "i": str(time.time()),
            "p": {},
        }
        self._set_attr_json("method", m)
        self.update()

    def close_view(self, text: str = ""):
        m = {
            "n": "closeView",
            "i": str(time.time()),
            "p": {"text": text},
        }
        self.value = text
        self._set_attr_json("method", m)
        self.update()

    # bar_leading
    @property
    def bar_leading(self) -> Optional[Control]:
        return self.__bar_leading

    @bar_leading.setter
    def bar_leading(self, value: Optional[Control]):
        self.__bar_leading = value

    # bar_trailing
    @property
    def bar_trailing(self) -> Optional[List[Control]]:
        return self.__bar_trailing

    @bar_trailing.setter
    def bar_trailing(self, value: Optional[List[Control]]):
        self.__bar_trailing = value if value is not None else []

    # bar_bgcolor
    @property
    def bar_bgcolor(self) -> ControlStateValue[str]:
        return self.__bar_bgcolor

    @bar_bgcolor.setter
    def bar_bgcolor(self, value: ControlStateValue[str]):
        self.__bar_bgcolor = value

    # bar_overlay_color
    @property
    def bar_overlay_color(self) -> ControlStateValue[str]:
        return self.__bar_overlay_color

    @bar_overlay_color.setter
    def bar_overlay_color(self, value: ControlStateValue[str]):
        self.__bar_overlay_color = value

    # bar_shadow_color
    @property
    def bar_shadow_color(self) -> Union[None, str, Dict[ControlState, str]]:
        return self.__bar_shadow_color

    @bar_shadow_color.setter
    def bar_shadow_color(self, value: Union[None, str, Dict[ControlState, str]]):
        self.__bar_shadow_color = value

    # bar_surface_tint_color
    @property
    def bar_surface_tint_color(self) -> Union[None, str, Dict[ControlState, str]]:
        return self.__bar_surface_tint_color

    @bar_surface_tint_color.setter
    def bar_surface_tint_color(self, value: Union[None, str, Dict[ControlState, str]]):
        self.__bar_surface_tint_color = value

    # bar_elevation
    @property
    def bar_elevation(self) -> Union[OptionalNumber, Dict[ControlState, Number]]:
        return self.__bar_elevation

    @bar_elevation.setter
    def bar_elevation(self, value: Union[OptionalNumber, Dict[ControlState, Number]]):
        self.__bar_elevation = value

    # bar_border_side
    @property
    def bar_border_side(
        self,
    ) -> ControlStateValue[BorderSide]:
        return self.__bar_border_side

    @bar_border_side.setter
    def bar_border_side(self, value: ControlStateValue[BorderSide]):
        self.__bar_border_side = value

    # bar_shape
    @property
    def bar_shape(
        self,
    ) -> ControlStateValue[OutlinedBorder]:
        return self.__bar_shape

    @bar_shape.setter
    def bar_shape(self, value: ControlStateValue[OutlinedBorder]):
        self.__bar_shape = value

    # bar_text_style
    @property
    def bar_text_style(self) -> ControlStateValue[TextStyle]:
        return self.__bar_text_style

    @bar_text_style.setter
    def bar_text_style(self, value: ControlStateValue[TextStyle]):
        self.__bar_text_style = value

    # bar_hint_text_style
    @property
    def bar_hint_text_style(
        self,
    ) -> ControlStateValue[TextStyle]:
        return self.__bar_hint_text_style

    @bar_hint_text_style.setter
    def bar_hint_text_style(self, value: ControlStateValue[TextStyle]):
        self.__bar_hint_text_style = value

    # bar_padding
    @property
    def bar_padding(self) -> ControlStateValue[PaddingValue]:
        return self.__bar_padding

    @bar_padding.setter
    def bar_padding(self, value: ControlStateValue[PaddingValue]):
        self.__bar_padding = value

    # view_leading
    @property
    def view_leading(self) -> Optional[Control]:
        return self.__view_leading

    @view_leading.setter
    def view_leading(self, value: Optional[Control]):
        self.__view_leading = value

    # view_surface_tint_color
    @property
    def view_surface_tint_color(self) -> Optional[ColorValue]:
        return self.__view_surface_tint_color

    @view_surface_tint_color.setter
    def view_surface_tint_color(self, value: Optional[ColorValue]):
        self.__view_surface_tint_color = value
        self._set_enum_attr("viewSurfaceTintColor", value, ColorEnums)

    # autofocus
    @property
    def autofocus(self) -> bool:
        return self._get_attr("autofocus", data_type="bool", def_value=False)

    @autofocus.setter
    def autofocus(self, value: Optional[bool]):
        self._set_attr("autofocus", value)

    # view_trailing
    @property
    def view_trailing(self) -> Optional[List[Control]]:
        return self.__view_trailing

    @view_trailing.setter
    def view_trailing(self, value: Optional[List[Control]]):
        self.__view_trailing = value if value is not None else []

    # view_elevation
    @property
    def view_elevation(self) -> OptionalNumber:
        return self._get_attr("viewElevation")

    @view_elevation.setter
    def view_elevation(self, value: OptionalNumber):
        self._set_attr("viewElevation", value)

    # view_header_height
    @property
    def view_header_height(self) -> OptionalNumber:
        return self._get_attr("viewHeaderHeight")

    @view_header_height.setter
    def view_header_height(self, value: OptionalNumber):
        self._set_attr("viewHeaderHeight", value)

    # view_bgcolor
    @property
    def view_bgcolor(self) -> Optional[str]:
        return self.__view_bgcolor

    @view_bgcolor.setter
    def view_bgcolor(self, value: Optional[str]):
        self.__view_bgcolor = value
        self._set_enum_attr("viewBgcolor", value, ColorEnums)

    # divider_color
    @property
    def divider_color(self) -> Optional[ColorValue]:
        return self.__divider_color

    @divider_color.setter
    def divider_color(self, value: Optional[ColorValue]):
        self.__divider_color = value
        self._set_enum_attr("dividerColor", value, ColorEnums)

    # bar_hint_text
    @property
    def bar_hint_text(self) -> Optional[str]:
        return self._get_attr("barHintText")

    @bar_hint_text.setter
    def bar_hint_text(self, value: Optional[str]):
        self._set_attr("barHintText", value)

    # view_hint_text
    @property
    def view_hint_text(self) -> Optional[str]:
        return self._get_attr("viewHintText")

    @view_hint_text.setter
    def view_hint_text(self, value: Optional[str]):
        self._set_attr("viewHintText", value)

    # view_shape
    @property
    def view_shape(self) -> Optional[OutlinedBorder]:
        return self.__view_shape

    @view_shape.setter
    def view_shape(self, value: Optional[OutlinedBorder]):
        self.__view_shape = value

    # view_side
    @property
    def view_side(self) -> Optional[BorderSide]:
        return self.__view_side

    @view_side.setter
    def view_side(self, value: Optional[BorderSide]):
        self.__view_side = value

    # view_size_constraints
    @property
    def view_size_constraints(self) -> Optional[BoxConstraints]:
        return self.__view_size_constraints

    @view_size_constraints.setter
    def view_size_constraints(self, value: Optional[BoxConstraints]):
        self.__view_size_constraints = value

    # bar_scroll_padding
    @property
    def bar_scroll_padding(self) -> Optional[PaddingValue]:
        return self.__bar_scroll_padding

    @bar_scroll_padding.setter
    def bar_scroll_padding(self, value: Optional[PaddingValue]):
        self.__bar_scroll_padding = value

    # full_screen
    @property
    def full_screen(self) -> bool:
        return self._get_attr("fullScreen", data_type="bool", def_value=False)

    @full_screen.setter
    def full_screen(self, value: Optional[bool]):
        self._set_attr("fullScreen", value)

    # capitalization
    @property
    def capitalization(self) -> TextCapitalization:
        return self.__capitalization

    @capitalization.setter
    def capitalization(self, value: TextCapitalization):
        self.__capitalization = value
        self._set_enum_attr("capitalization", value, TextCapitalization)

    # keyboard_type
    @property
    def keyboard_type(self) -> KeyboardType:
        return self.__keyboard_type

    @keyboard_type.setter
    def keyboard_type(self, value: KeyboardType):
        self.__keyboard_type = value
        self._set_enum_attr("keyboardType", value, KeyboardType)

    # view_header_text_style
    @property
    def view_header_text_style(self):
        return self.__view_header_text_style

    @view_header_text_style.setter
    def view_header_text_style(self, value: Optional[TextStyle]):
        self.__view_header_text_style = value

    # view_hint_text_style
    @property
    def view_hint_text_style(self):
        return self.__view_hint_text_style

    @view_hint_text_style.setter
    def view_hint_text_style(self, value: Optional[TextStyle]):
        self.__view_hint_text_style = value

    # controls
    @property
    def controls(self):
        return self.__controls

    @controls.setter
    def controls(self, value: Optional[Sequence[Control]]):
        self.__controls = list(value) if value is not None else []

    # value
    @property
    def value(self) -> Optional[str]:
        return self._get_attr("value", def_value="")

    @value.setter
    def value(self, value: Optional[str]):
        self._set_attr("value", value)

    # on_change
    @property
    def on_change(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change")

    @on_change.setter
    def on_change(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change", handler)
        self._set_attr("onchange", True if handler is not None else None)

    # on_focus
    @property
    def on_focus(self) -> OptionalControlEventCallable:
        return self._get_event_handler("focus")

    @on_focus.setter
    def on_focus(self, handler: OptionalControlEventCallable):
        self._add_event_handler("focus", handler)

    # on_blur
    @property
    def on_blur(self) -> OptionalControlEventCallable:
        return self._get_event_handler("blur")

    @on_blur.setter
    def on_blur(self, handler: OptionalControlEventCallable):
        self._add_event_handler("blur", handler)

    # on_tap
    @property
    def on_tap(self) -> OptionalControlEventCallable:
        return self._get_event_handler("tap")

    @on_tap.setter
    def on_tap(self, handler: OptionalControlEventCallable):
        self._add_event_handler("tap", handler)
        self._set_attr("ontap", True if handler is not None else None)

    # on_tap_outside_bar
    @property
    def on_tap_outside_bar(self) -> OptionalControlEventCallable:
        return self._get_event_handler("tapOutsideBar")

    @on_tap_outside_bar.setter
    def on_tap_outside_bar(self, handler: OptionalControlEventCallable):
        self._add_event_handler("tapOutsideBar", handler)
        self._set_attr("onTapOutsideBar", True if handler is not None else None)

    # on_submit
    @property
    def on_submit(self) -> OptionalControlEventCallable:
        return self._get_event_handler("submit")

    @on_submit.setter
    def on_submit(self, handler: OptionalControlEventCallable):
        self._add_event_handler("submit", handler)
        self._set_attr("onsubmit", True if handler is not None else None)



from typing import Any, List, Optional

from flet.core.alignment import Alignment
from flet.core.canvas.shape import Shape
from flet.core.control import OptionalNumber
from flet.core.inline_span import InlineSpan
from flet.core.text_style import TextStyle
from flet.core.types import TextAlign


class Text(Shape):
    def __init__(
        self,
        x: OptionalNumber = None,
        y: OptionalNumber = None,
        text: Optional[str] = None,
        style: Optional[TextStyle] = None,
        spans: Optional[List[InlineSpan]] = None,
        alignment: Optional[Alignment] = None,
        text_align: Optional[TextAlign] = None,
        max_lines: Optional[int] = None,
        max_width: OptionalNumber = None,
        ellipsis: Optional[str] = None,
        rotate: OptionalNumber = None,
        #
        # Control
        #
        ref=None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        Shape.__init__(self, ref=ref, visible=visible, disabled=disabled, data=data)

        self.x = x
        self.y = y
        self.text = text
        self.style = style
        self.spans = spans
        self.alignment = alignment
        self.text_align = text_align
        self.max_lines = max_lines
        self.max_width = max_width
        self.ellipsis = ellipsis
        self.rotate = rotate

    def _get_control_name(self):
        return "text"

    def _get_children(self):
        children = []
        children.extend(self.__spans)
        return children

    def before_update(self):
        super().before_update()
        self._set_attr_json("style", self.__style)
        self._set_attr_json("alignment", self.__alignment)

    # x
    @property
    def x(self) -> OptionalNumber:
        return self._get_attr("x")

    @x.setter
    def x(self, value: OptionalNumber):
        self._set_attr("x", value)

    # y
    @property
    def y(self) -> OptionalNumber:
        return self._get_attr("y")

    @y.setter
    def y(self, value: OptionalNumber):
        self._set_attr("y", value)

    # text
    @property
    def text(self) -> Optional[str]:
        return self._get_attr("text")

    @text.setter
    def text(self, value: Optional[str]):
        self._set_attr("text", value)

    # style
    @property
    def style(self) -> Optional[TextStyle]:
        return self.__style

    @style.setter
    def style(self, value: Optional[TextStyle]):
        self.__style = value

    # spans
    @property
    def spans(self) -> Optional[List[InlineSpan]]:
        return self.__spans

    @spans.setter
    def spans(self, value: Optional[List[InlineSpan]]):
        self.__spans = value if value is not None else []

    # alignment
    @property
    def alignment(self) -> Optional[Alignment]:
        return self.__alignment

    @alignment.setter
    def alignment(self, value: Optional[Alignment]):
        self.__alignment = value

    # text_align
    @property
    def text_align(self) -> Optional[TextAlign]:
        return self.__text_align

    @text_align.setter
    def text_align(self, value: Optional[TextAlign]):
        self.__text_align = value
        self._set_attr(
            "textAlign", value.value if isinstance(value, TextAlign) else value
        )

    # max_lines
    @property
    def max_lines(self) -> Optional[int]:
        return self._get_attr("maxLines")

    @max_lines.setter
    def max_lines(self, value: Optional[int]):
        self._set_attr("maxLines", value)

    # max_width
    @property
    def max_width(self) -> OptionalNumber:
        return self._get_attr("maxWidth")

    @max_width.setter
    def max_width(self, value: OptionalNumber):
        self._set_attr("maxWidth", value)

    # ellipsis
    @property
    def ellipsis(self) -> Optional[str]:
        return self._get_attr("ellipsis")

    @ellipsis.setter
    def ellipsis(self, value: Optional[str]):
        self._set_attr("ellipsis", value)

    # rotate
    @property
    def rotate(self) -> OptionalNumber:
        return self._get_attr("rotate")

    @rotate.setter
    def rotate(self, value: OptionalNumber):
        self._set_attr("rotate", value)



from typing import Any, Optional, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ColorEnums,
    ColorValue,
    OffsetValue,
    OptionalControlEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
)


class CupertinoSlider(ConstrainedControl):
    """
    An iOS-type slider.

    It provides a visual indication of adjustable content, as well as the current setting in the total range of content.

    Use a slider when you want people to set defined values (such as volume or brightness), or when people would benefit from instant feedback on the effect of setting changes.

    -----

    Online docs: https://flet.dev/docs/controls/cupertinoslider
    """

    def __init__(
        self,
        value: OptionalNumber = None,
        min: OptionalNumber = None,
        max: OptionalNumber = None,
        divisions: Optional[int] = None,
        active_color: Optional[ColorValue] = None,
        thumb_color: Optional[ColorValue] = None,
        on_change: OptionalControlEventCallable = None,
        on_change_start: OptionalControlEventCallable = None,
        on_change_end: OptionalControlEventCallable = None,
        on_focus: OptionalControlEventCallable = None,
        on_blur: OptionalControlEventCallable = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )
        self.value = value
        self.min = min
        self.max = max
        self.divisions = divisions
        self.round = round
        self.active_color = active_color
        self.thumb_color = thumb_color
        self.on_change = on_change
        self.on_change_start = on_change_start
        self.on_change_end = on_change_end
        self.on_focus = on_focus
        self.on_blur = on_blur

    def _get_control_name(self):
        return "cupertinoslider"

    def before_update(self):
        super().before_update()
        assert (
            self.min is None or self.max is None or self.min <= self.max
        ), "min must be less than or equal to max"
        assert (
            self.min is None or self.value is None or (self.value >= self.min)
        ), "value must be greater than or equal to min"
        assert (
            self.max is None or self.value is None or (self.value <= self.max)
        ), "value must be less than or equal to max"

    # value
    @property
    def value(self) -> float:
        return self._get_attr("value", data_type="float", def_value=self.min or 0.0)

    @value.setter
    def value(self, value: OptionalNumber):
        self._set_attr("value", value)

    # min
    @property
    def min(self) -> float:
        return self._get_attr("min", data_type="float", def_value=0.0)

    @min.setter
    def min(self, value: OptionalNumber):
        self._set_attr("min", value)

    # max
    @property
    def max(self) -> float:
        return self._get_attr("max", data_type="float", def_value=1.0)

    @max.setter
    def max(self, value: OptionalNumber):
        self._set_attr("max", value)

    # divisions
    @property
    def divisions(self) -> Optional[int]:
        return self._get_attr("divisions", data_type="int")

    @divisions.setter
    def divisions(self, value: Optional[int]):
        self._set_attr("divisions", value)

    # round
    @property
    def round(self) -> int:
        return self._get_attr("round", data_type="int", def_value=0)

    @round.setter
    def round(self, value: Optional[int]):
        self._set_attr("round", value)

    # active_color
    @property
    def active_color(self) -> Optional[ColorValue]:
        return self.__active_color

    @active_color.setter
    def active_color(self, value: Optional[ColorValue]):
        self.__active_color = value
        self._set_enum_attr("activeColor", value, ColorEnums)

    # thumb_color
    @property
    def thumb_color(self) -> Optional[ColorValue]:
        return self.__thumb_color

    @thumb_color.setter
    def thumb_color(self, value: Optional[ColorValue]):
        self.__thumb_color = value
        self._set_enum_attr("thumbColor", value, ColorEnums)

    # on_change
    @property
    def on_change(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change")

    @on_change.setter
    def on_change(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change", handler)

    # on_change_start
    @property
    def on_change_start(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change_start")

    @on_change_start.setter
    def on_change_start(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change_start", handler)

    # on_change_end
    @property
    def on_change_end(self) -> OptionalControlEventCallable:
        return self._get_event_handler("change_end")

    @on_change_end.setter
    def on_change_end(self, handler: OptionalControlEventCallable):
        self._add_event_handler("change_end", handler)

    # on_focus
    @property
    def on_focus(self) -> OptionalControlEventCallable:
        return self._get_event_handler("focus")

    @on_focus.setter
    def on_focus(self, handler: OptionalControlEventCallable):
        self._add_event_handler("focus", handler)

    # on_blur
    @property
    def on_blur(self) -> OptionalControlEventCallable:
        return self._get_event_handler("blur")

    @on_blur.setter
    def on_blur(self, handler: OptionalControlEventCallable):
        self._add_event_handler("blur", handler)



from typing import Any, Optional, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.buttons import OutlinedBorder
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    ClipBehavior,
    ColorEnums,
    ColorValue,
    IconEnums,
    IconValue,
    MouseCursor,
    OffsetValue,
    OptionalControlEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
    UrlTarget,
)


class FloatingActionButton(ConstrainedControl):
    """
    A floating action button is a circular icon button that hovers over content to promote a primary action in the application. Floating action button is usually set to `page.floating_action_button`, but can also be added as a regular control at any place on a page.

    Example:
    ```
    import flet as ft

    def main(page: ft.Page):
        page.title = "Floating Action Button"
        page.theme_mode = ft.ThemeMode.LIGHT
        page.horizontal_alignment = ft.CrossAxisAlignment.CENTER
        page.auto_scroll = True
        page.scroll = ft.ScrollMode.HIDDEN
        page.appbar = ft.AppBar(
            title=ft.Text(
                "Floating Action Button", weight=ft.FontWeight.BOLD, color=ft.colors.BLACK87
            ),
            bgcolor=ft.colors.BLUE,
            center_title=True,
            actions=[
                ft.IconButton(ft.icons.MENU, tooltip="Menu", icon_color=ft.colors.BLACK87)
            ],
            color=ft.colors.WHITE,
        )

        # keeps track of the number of tiles already added
        page.count = 0

        def fab_pressed(e):
            page.add(ft.ListTile(title=ft.Text(f"Tile {page.count}")))
            page.show_snack_bar(
                ft.SnackBar(ft.Text("Tile was added successfully!"), open=True)
            )
            page.count += 1

        page.floating_action_button = ft.FloatingActionButton(
            icon=ft.icons.ADD, on_click=fab_pressed, bgcolor=ft.colors.LIME_300
        )
        page.add(ft.Text("Press the FAB to add a tile!"))

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/floatingactionbutton
    """

    def __init__(
        self,
        text: Optional[str] = None,
        icon: Optional[IconValue] = None,
        bgcolor: Optional[ColorValue] = None,
        content: Optional[Control] = None,
        shape: Optional[OutlinedBorder] = None,
        autofocus: Optional[bool] = None,
        mini: Optional[bool] = None,
        foreground_color: Optional[ColorValue] = None,
        focus_color: Optional[ColorValue] = None,
        clip_behavior: Optional[ClipBehavior] = None,
        elevation: OptionalNumber = None,
        disabled_elevation: OptionalNumber = None,
        focus_elevation: OptionalNumber = None,
        highlight_elevation: OptionalNumber = None,
        hover_elevation: OptionalNumber = None,
        enable_feedback: Optional[bool] = None,
        url: Optional[str] = None,
        url_target: Optional[UrlTarget] = None,
        mouse_cursor: Optional[MouseCursor] = None,
        on_click: OptionalControlEventCallable = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        self.text = text
        self.icon = icon
        self.bgcolor = bgcolor
        self.content = content
        self.autofocus = autofocus
        self.shape = shape
        self.mini = mini
        self.url = url
        self.url_target = url_target
        self.on_click = on_click
        self.foreground_color = foreground_color
        self.focus_color = focus_color
        self.clip_behavior = clip_behavior
        self.elevation = elevation
        self.disabled_elevation = disabled_elevation
        self.focus_elevation = focus_elevation
        self.highlight_elevation = highlight_elevation
        self.hover_elevation = hover_elevation
        self.enable_feedback = enable_feedback
        self.mouse_cursor = mouse_cursor

    def _get_control_name(self):
        return "floatingactionbutton"

    def before_update(self):
        super().before_update()
        assert (
            self.text or self.icon or (self.__content and self.__content.visible)
        ), "at minimum, text, icon or a visible content must be provided"
        self._set_attr_json("shape", self.__shape)

    def _get_children(self):
        if self.__content is None:
            return []
        self.__content._set_attr_internal("n", "content")
        return [self.__content]

    # text
    @property
    def text(self) -> Optional[str]:
        return self._get_attr("text")

    @text.setter
    def text(self, value: Optional[str]):
        self._set_attr("text", value)

    # icon
    @property
    def icon(self) -> Optional[IconValue]:
        return self.__icon

    @icon.setter
    def icon(self, value: Optional[IconValue]):
        self.__icon = value
        self._set_enum_attr("icon", value, IconEnums)

    # bgcolor
    @property
    def bgcolor(self) -> Optional[ColorValue]:
        return self.__bgcolor

    @bgcolor.setter
    def bgcolor(self, value: Optional[ColorValue]):
        self.__bgcolor = value
        self._set_enum_attr("bgcolor", value, ColorEnums)

    # url
    @property
    def url(self) -> Optional[str]:
        return self._get_attr("url")

    @url.setter
    def url(self, value: Optional[str]):
        self._set_attr("url", value)

    # url_target
    @property
    def url_target(self) -> Optional[UrlTarget]:
        return self.__url_target

    @url_target.setter
    def url_target(self, value: Optional[UrlTarget]):
        self.__url_target = value
        self._set_enum_attr("urlTarget", value, UrlTarget)

    # mouse_cursor
    @property
    def mouse_cursor(self) -> Optional[MouseCursor]:
        return self.__mouse_cursor

    @mouse_cursor.setter
    def mouse_cursor(self, value: Optional[MouseCursor]):
        self.__mouse_cursor = value
        self._set_enum_attr("mouseCursor", value, MouseCursor)

    # on_click
    @property
    def on_click(self) -> OptionalControlEventCallable:
        return self._get_event_handler("click")

    @on_click.setter
    def on_click(self, handler: OptionalControlEventCallable):
        self._add_event_handler("click", handler)

    # content
    @property
    def content(self) -> Optional[Control]:
        return self.__content

    @content.setter
    def content(self, value: Optional[Control]):
        self.__content = value

    # autofocus
    @property
    def autofocus(self) -> bool:
        return self._get_attr("autofocus", data_type="bool", def_value=False)

    @autofocus.setter
    def autofocus(self, value: Optional[bool]):
        self._set_attr("autofocus", value)

    # shape
    @property
    def shape(self) -> Optional[OutlinedBorder]:
        return self.__shape

    @shape.setter
    def shape(self, value: Optional[OutlinedBorder]):
        self.__shape = value

    # mini
    @property
    def mini(self) -> bool:
        return self._get_attr("mini", data_type="bool", def_value=False)

    @mini.setter
    def mini(self, value: Optional[bool]):
        self._set_attr("mini", value)

    # elevation
    @property
    def elevation(self) -> OptionalNumber:
        return self._get_attr("elevation", data_type="float")

    @elevation.setter
    def elevation(self, value: OptionalNumber):
        assert value is None or value >= 0, "elevation cannot be negative"
        self._set_attr("elevation", value)

    # disabled_elevation
    @property
    def disabled_elevation(self) -> OptionalNumber:
        return self._get_attr("disabledElevation", data_type="float")

    @disabled_elevation.setter
    def disabled_elevation(self, value: OptionalNumber):
        assert value is None or value >= 0, "disabled_elevation cannot be negative"
        self._set_attr("disabledElevation", value)

    # enable_feedback
    @property
    def enable_feedback(self) -> bool:
        return self._get_attr("enableFeedback", data_type="bool", def_value=True)

    @enable_feedback.setter
    def enable_feedback(self, value: Optional[bool]):
        self._set_attr("enableFeedback", value)

    # focus_color
    @property
    def focus_color(self) -> Optional[ColorValue]:
        return self.__focus_color

    @focus_color.setter
    def focus_color(self, value: Optional[ColorValue]):
        self.__focus_color = value
        self._set_enum_attr("focusColor", value, ColorEnums)

    # focus_elevation
    @property
    def focus_elevation(self) -> OptionalNumber:
        return self._get_attr("focusElevation", data_type="float")

    @focus_elevation.setter
    def focus_elevation(self, value: OptionalNumber):
        assert value is None or value >= 0, "focus_elevation cannot be negative"
        self._set_attr("focusElevation", value)

    # foreground_color
    @property
    def foreground_color(self) -> Optional[ColorValue]:
        return self.__foreground_color

    @foreground_color.setter
    def foreground_color(self, value: Optional[ColorValue]):
        self.__foreground_color = value
        self._set_enum_attr("foregroundColor", value, ColorEnums)

    # highlight_elevation
    @property
    def highlight_elevation(self) -> OptionalNumber:
        return self._get_attr("highlightElevation", data_type="float")

    @highlight_elevation.setter
    def highlight_elevation(self, value: OptionalNumber):
        assert value is None or value >= 0, "highlight_elevation cannot be negative"
        self._set_attr("highlightElevation", value)

    # hover_elevation
    @property
    def hover_elevation(self) -> OptionalNumber:
        return self._get_attr("hoverElevation", data_type="float")

    @hover_elevation.setter
    def hover_elevation(self, value: OptionalNumber):
        assert value is None or value >= 0, "hover_elevation cannot be negative"
        self._set_attr("hoverElevation", value)

    # clip_behavior
    @property
    def clip_behavior(self) -> Optional[ClipBehavior]:
        return self.__clip_behavior

    @clip_behavior.setter
    def clip_behavior(self, value: Optional[ClipBehavior]):
        self.__clip_behavior = value
        self._set_enum_attr("clipBehavior", value, ClipBehavior)



from typing import Any, Optional, Union

from flet.core.animation import AnimationValue
from flet.core.badge import BadgeValue
from flet.core.box import FilterQuality
from flet.core.constrained_control import ConstrainedControl
from flet.core.control import Control, OptionalNumber
from flet.core.ref import Ref
from flet.core.tooltip import TooltipValue
from flet.core.types import (
    BlendMode,
    BorderRadiusValue,
    ColorEnums,
    ColorValue,
    ImageFit,
    ImageRepeat,
    OffsetValue,
    OptionalControlEventCallable,
    ResponsiveNumber,
    RotateValue,
    ScaleValue,
)

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal


class Image(ConstrainedControl):
    """
    A control that displays an image.

    Example:
    ```
    import flet as ft

    def main(page: ft.Page):
        page.title = "Image Example"

        img = ft.Image(
            src=f"/icons/icon-512.png",
            width=100,
            height=100,
            fit=ft.ImageFit.CONTAIN,
        )

        page.add(img)

    ft.app(target=main)
    ```

    -----

    Online docs: https://flet.dev/docs/controls/image
    """

    def __init__(
        self,
        src: Optional[str] = None,
        src_base64: Optional[str] = None,
        error_content: Optional[Control] = None,
        repeat: Optional[ImageRepeat] = None,
        fit: Optional[ImageFit] = None,
        border_radius: Optional[BorderRadiusValue] = None,
        color: Optional[ColorValue] = None,
        color_blend_mode: Optional[BlendMode] = None,
        gapless_playback: Optional[bool] = None,
        semantics_label: Optional[str] = None,
        exclude_from_semantics: Optional[bool] = None,
        filter_quality: Optional[FilterQuality] = None,
        cache_width: Optional[int] = None,
        cache_height: Optional[int] = None,
        anti_alias: Optional[bool] = None,
        #
        # ConstrainedControl
        #
        ref: Optional[Ref] = None,
        key: Optional[str] = None,
        width: OptionalNumber = None,
        height: OptionalNumber = None,
        left: OptionalNumber = None,
        top: OptionalNumber = None,
        right: OptionalNumber = None,
        bottom: OptionalNumber = None,
        expand: Union[None, bool, int] = None,
        expand_loose: Optional[bool] = None,
        col: Optional[ResponsiveNumber] = None,
        opacity: OptionalNumber = None,
        rotate: Optional[RotateValue] = None,
        scale: Optional[ScaleValue] = None,
        offset: Optional[OffsetValue] = None,
        aspect_ratio: OptionalNumber = None,
        animate_opacity: Optional[AnimationValue] = None,
        animate_size: Optional[AnimationValue] = None,
        animate_position: Optional[AnimationValue] = None,
        animate_rotation: Optional[AnimationValue] = None,
        animate_scale: Optional[AnimationValue] = None,
        animate_offset: Optional[AnimationValue] = None,
        on_animation_end: OptionalControlEventCallable = None,
        tooltip: Optional[TooltipValue] = None,
        badge: Optional[BadgeValue] = None,
        visible: Optional[bool] = None,
        disabled: Optional[bool] = None,
        data: Any = None,
    ):
        ConstrainedControl.__init__(
            self,
            ref=ref,
            key=key,
            width=width,
            height=height,
            left=left,
            top=top,
            right=right,
            bottom=bottom,
            expand=expand,
            expand_loose=expand_loose,
            col=col,
            opacity=opacity,
            rotate=rotate,
            scale=scale,
            offset=offset,
            aspect_ratio=aspect_ratio,
            animate_opacity=animate_opacity,
            animate_size=animate_size,
            animate_position=animate_position,
            animate_rotation=animate_rotation,
            animate_scale=animate_scale,
            animate_offset=animate_offset,
            on_animation_end=on_animation_end,
            tooltip=tooltip,
            badge=badge,
            visible=visible,
            disabled=disabled,
            data=data,
        )

        self.src = src
        self.src_base64 = src_base64
        self.error_content = error_content
        self.fit = fit
        self.repeat = repeat
        self.border_radius = border_radius
        self.color = color
        self.color_blend_mode = color_blend_mode
        self.gapless_playback = gapless_playback
        self.semantics_label = semantics_label
        self.exclude_from_semantics = exclude_from_semantics
        self.filter_quality = filter_quality
        self.cache_width = cache_width
        self.cache_height = cache_height
        self.anti_alias = anti_alias

    def _get_control_name(self):
        return "image"

    def _get_children(self):
        if self.__error_content is not None:
            self.__error_content._set_attr_internal("n", "error_content")
            return [self.__error_content]
        return []

    def before_update(self):
        super().before_update()
        self._set_attr_json("borderRadius", self.__border_radius)

    # src
    @property
    def src(self):
        return self._get_attr("src")

    @src.setter
    def src(self, value: Optional[str]):
        self._set_attr("src", value)

    # src_base64
    @property
    def src_base64(self):
        return self._get_attr("srcBase64")

    @src_base64.setter
    def src_base64(self, value: Optional[str]):
        self._set_attr("srcBase64", value)

    # fit
    @property
    def fit(self) -> Optional[ImageFit]:
        return self.__fit

    @fit.setter
    def fit(self, value: Optional[ImageFit]):
        self.__fit = value
        self._set_attr("fit", value.value if isinstance(value, ImageFit) else value)

    # filter_quality
    @property
    def filter_quality(self) -> Optional[FilterQuality]:
        return self.__filter_quality

    @filter_quality.setter
    def filter_quality(self, value: Optional[FilterQuality]):
        self.__filter_quality = value
        self._set_attr(
            "filterQuality", value.value if isinstance(value, FilterQuality) else value
        )

    # repeat
    @property
    def repeat(self) -> Optional[ImageRepeat]:
        return self.__repeat

    @repeat.setter
    def repeat(self, value: Optional[ImageRepeat]):
        self.__repeat = value
        self._set_enum_attr("repeat", value, ImageRepeat)

    # cache_width
    @property
    def cache_width(self) -> Optional[int]:
        return self._get_attr("cacheWidth", data_type="int")

    @cache_width.setter
    def cache_width(self, value: Optional[int]):
        self._set_attr("cacheWidth", value)

    # cache_height
    @property
    def cache_height(self) -> Optional[int]:
        return self._get_attr("cacheHeight", data_type="int")

    @cache_height.setter
    def cache_height(self, value: Optional[int]):
        self._set_attr("cacheHeight", value)

    # anti_alias
    @property
    def anti_alias(self) -> Optional[bool]:
        return self._get_attr("antiAlias", data_type="bool", def_value=False)

    @anti_alias.setter
    def anti_alias(self, value: Optional[bool]):
        self._set_attr("antiAlias", value)

    # border_radius
    @property
    def border_radius(self) -> Optional[BorderRadiusValue]:
        return self.__border_radius

    @border_radius.setter
    def border_radius(self, value: Optional[BorderRadiusValue]):
        self.__border_radius = value

    # color
    @property
    def color(self) -> Optional[ColorValue]:
        return self.__color

    @color.setter
    def color(self, value: Optional[ColorValue]):
        self.__color = value
        self._set_enum_attr("color", value, ColorEnums)

    # color_blend_mode
    @property
    def color_blend_mode(self) -> Optional[BlendMode]:
        return self.__blend_mode

    @color_blend_mode.setter
    def color_blend_mode(self, value: Optional[BlendMode]):
        self.__blend_mode = value
        self._set_enum_attr("colorBlendMode", value, BlendMode)

    # gapless_playback
    @property
    def gapless_playback(self) -> bool:
        return self._get_attr("gaplessPlayback", data_type="bool", def_value=False)

    @gapless_playback.setter
    def gapless_playback(self, value: Optional[bool]):
        self._set_attr("gaplessPlayback", value)

    # exclude_from_semantics
    @property
    def exclude_from_semantics(self) -> bool:
        return self._get_attr("excludeFromSemantics", data_type="bool", def_value=False)

    @exclude_from_semantics.setter
    def exclude_from_semantics(self, value: Optional[bool]):
        self._set_attr("excludeFromSemantics", value)

    # semantics_label
    @property
    def semantics_label(self) -> Optional[str]:
        return self._get_attr("semanticsLabel")

    @semantics_label.setter
    def semantics_label(self, value: Optional[str]):
        self._set_attr("semanticsLabel", value)

    # error_content
    @property
    def error_content(self) -> Optional[Control]:
        return self.__error_content

    @error_content.setter
    def error_content(self, value: Optional[Control]):
        self.__error_content = value



from typing import Optional

from flet.auth.oauth_provider import OAuthProvider


class AzureOAuthProvider(OAuthProvider):
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        redirect_url: str,
        tenant: Optional[str] = "common",
    ) -> None:
        super().__init__(
            client_id=client_id,
            client_secret=client_secret,
            authorization_endpoint=f"https://login.microsoftonline.com/{tenant}/oauth2/v2.0/authorize",
            token_endpoint=f"https://login.microsoftonline.com/{tenant}/oauth2/v2.0/token",
            redirect_url=redirect_url,
            user_scopes=["user.read"],
            user_endpoint="https://graph.microsoft.com/v1.0/me",
            user_id_fn=lambda u: u["id"],
            group_scopes=[],
        )
        self.tenant = tenant



import json
from typing import List, Optional

import httpx
from flet.auth.group import Group
from flet.auth.oauth_provider import OAuthProvider
from flet.auth.user import User
from flet.version import version


class GitHubOAuthProvider(OAuthProvider):
    def __init__(self, client_id: str, client_secret: str, redirect_url: str) -> None:
        super().__init__(
            client_id=client_id,
            client_secret=client_secret,
            authorization_endpoint="https://github.com/login/oauth/authorize",
            token_endpoint="https://github.com/login/oauth/access_token",
            redirect_url=redirect_url,
            user_scopes=["read:user", "user:email"],
            group_scopes=["read:org"],
        )

    def _fetch_groups(self, access_token: str) -> List[Group]:
        with httpx.Client(follow_redirects=True) as client:
            teams_resp = client.send(self.__get_user_teams_request(access_token))
            return self.__complete_fetch_groups(teams_resp)

    async def _fetch_groups_async(self, access_token: str) -> List[Group]:
        async with httpx.AsyncClient(follow_redirects=True) as client:
            teams_resp = await client.send(self.__get_user_teams_request(access_token))
            return self.__complete_fetch_groups(teams_resp)

    def __get_user_teams_request(self, access_token):
        return httpx.Request(
            "GET",
            "https://api.github.com/user/teams",
            headers=self.__get_client_headers(access_token),
        )

    def __complete_fetch_groups(self, teams_resp):
        teams_resp.raise_for_status()
        groups = []
        tj = json.loads(teams_resp.text)
        for t in tj:
            groups.append(
                Group(
                    t,
                    name=t["name"],
                )
            )
        return groups

    def _fetch_user(self, access_token: str) -> Optional[User]:
        user_req, emails_req = self.__get_user_details_requests(access_token)
        with httpx.Client(follow_redirects=True) as client:
            user_resp = client.send(user_req)
            emails_resp = client.send(emails_req)
            return self.__complete_fetch_user_details(user_resp, emails_resp)

    async def _fetch_user_async(self, access_token: str) -> Optional[User]:
        user_req, emails_req = self.__get_user_details_requests(access_token)
        async with httpx.AsyncClient(follow_redirects=True) as client:
            user_resp = await client.send(user_req)
            emails_resp = await client.send(emails_req)
            return self.__complete_fetch_user_details(user_resp, emails_resp)

    def __get_user_details_requests(self, access_token):
        return (
            httpx.Request(
                "GET",
                "https://api.github.com/user",
                headers=self.__get_client_headers(access_token),
            ),
            httpx.Request(
                "GET",
                "https://api.github.com/user/emails",
                headers=self.__get_client_headers(access_token),
            ),
        )

    def __complete_fetch_user_details(self, user_resp, emails_resp):
        user_resp.raise_for_status()
        emails_resp.raise_for_status()
        uj = json.loads(user_resp.text)
        ej = json.loads(emails_resp.text)
        for e in ej:
            if e["primary"]:
                uj["email"] = e["email"]
                break
        return User(uj, id=str(uj["id"]))

    def __get_client_headers(self, access_token):
        return {
            "Authorization": f"Bearer {access_token}",
            "User-Agent": f"Flet/{version}",
        }
